@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -402,6 +402,10 @@ static int parse_arguments(int                          argc,
                     {
                         trans_a = HIPBLAS_OP_T;
                     }
+                    else if(strncmp(argv[i], "C", 1) == 0 || strncmp(argv[i], "c", 1) == 0)
+                    {
+                        trans_a = HIPBLAS_OP_C;
+                    }
                     else
                     {
                         std::cerr << "error with " << arg << std::endl;
@@ -420,6 +424,10 @@ static int parse_arguments(int                          argc,
                     {
                         trans_b = HIPBLAS_OP_T;
                     }
+                    else if(strncmp(argv[i], "C", 1) == 0 || strncmp(argv[i], "c", 1) == 0)
+                    {
+                        trans_b = HIPBLAS_OP_C;
+                    }
                     else
                     {
                         std::cerr << "error with " << arg << std::endl;
@@ -510,7 +518,7 @@ bool bad_argument(hipblasOperation_t trans_a,
         argument_error = true;
         std::cerr << "ERROR: bad argument lda = " << lda << " < " << m << std::endl;
     }
-    if((trans_a == HIPBLAS_OP_T) && (lda < k))
+    if((trans_a != HIPBLAS_OP_N) && (lda < k))
     {
         argument_error = true;
         std::cerr << "ERROR: bad argument lda = " << lda << " < " << k << std::endl;
@@ -520,7 +528,7 @@ bool bad_argument(hipblasOperation_t trans_a,
         argument_error = true;
         std::cerr << "ERROR: bad argument ldb = " << ldb << " < " << k << std::endl;
     }
-    if((trans_b == HIPBLAS_OP_T) && (ldb < n))
+    if((trans_b != HIPBLAS_OP_N) && (ldb < n))
     {
         argument_error = true;
         std::cerr << "ERROR: bad argument ldb = " << ldb << " < " << n << std::endl;
@@ -770,7 +778,7 @@ int test_hipblaslt(hipDataType                 in_datatype,
     }
 
     // Set User Preference attributes
-    uint64_t max_workspace_size = 32 * 1024 * 1024;
+    uint64_t max_workspace_size = 128 * 1024 * 1024;
     void*    d_workspace;
     CHECK_HIP_ERROR(hipMalloc(&d_workspace, max_workspace_size));
 
@@ -811,8 +819,8 @@ int test_hipblaslt(hipDataType                 in_datatype,
 
     for(int i = 0; i < gemm_count; i++)
     {
-        std::cout << i << ", " << (trans_a == HIPBLAS_OP_N ? "N" : "T")
-                  << (trans_b == HIPBLAS_OP_N ? "N" : "T") << ", " << m[i] << ", " << n[i] << ", "
+        std::cout << i << ", " << hipblas_operation_to_string(trans_a)
+                  << hipblas_operation_to_string(trans_b) << ", " << m[i] << ", " << n[i] << ", "
                   << k[i] << ", " << lda[i] << ", " << ldb[i] << ", " << ldc[i] << ", "
                   << stride_a[i] << ", " << stride_b[i] << ", " << stride_c[i] << ", "
                   << batch_count[i] << ", " << alpha[i] << ", " << beta[i] << ", " << enable_bias[i]
@@ -887,7 +895,21 @@ int test_hipblaslt(hipDataType                 in_datatype,
     if(validIdx.empty())
     {
         std::cerr << "No Solution found!" << std::endl;
-        return EXIT_FAILURE;
+        CHECK_HIP_ERROR(hipFree(d_n_vec));
+        CHECK_HIP_ERROR(hipFree(d_workspace));
+        CHECK_HIPBLASLT_ERROR(hipblasLtDestroy(handle));
+
+        for(int i = 0; i < gemm_count; i++)
+        {
+            CHECK_HIP_ERROR(hipFree(da[i]));
+            CHECK_HIP_ERROR(hipFree(db[i]));
+            CHECK_HIP_ERROR(hipFree(dc[i]));
+            CHECK_HIP_ERROR(hipFree(dd[i]));
+            if(enable_bias[i])
+                CHECK_HIP_ERROR(hipFree(d_bias[i]));
+        }
+        CHECK_HIP_ERROR(hipStreamDestroy(stream));
+        return EXIT_SUCCESS;
     }
 
     // step2: get default DeviceUserArguments from grouped gemm objects
