@@ -27,6 +27,8 @@
 #pragma once
 
 #include "hipblaslt_arguments.hpp"
+#include <fstream>
+#include <string>
 
 namespace ArgumentLogging
 {
@@ -84,7 +86,7 @@ public:
 
         // per/us to per/sec *10^6
         double hipblaslt_gflops = gflops * batch_count / gpu_us * 1e6;
-        double hipblaslt_GBps   = gbytes * batch_count / gpu_us * 1e6;
+        double hipblaslt_GBps   = gbytes / gpu_us * 1e6;
 
         // append performance fields
         if(gflops != ArgumentLogging::NA_value)
@@ -131,7 +133,8 @@ public:
                 {
                     name_line << ",atol";
                     if(atol == 1) // atol == init value
-                        val_line << "," << "failed";
+                        val_line << ","
+                                 << "failed";
                     else
                         val_line << "," << atol;
                 }
@@ -139,7 +142,8 @@ public:
                 {
                     name_line << ",rtol";
                     if(rtol == 1) // rtol == init value
-                        val_line << "," << "failed";
+                        val_line << ","
+                                 << "failed";
                     else
                         val_line << "," << rtol;
                 }
@@ -147,23 +151,25 @@ public:
         }
     }
 
-    template <typename T>
-    void log_args(hipblaslt_internal_ostream& str,
+    void log_args(hipDataType                 Tc,
+                  hipblaslt_internal_ostream& str,
                   size_t                      index,
                   int32_t                     solution_index,
                   std::string&                solution_name,
                   std::string&                kernel_name,
+                  std::string&                archName,
+                  std::string&                cuNum,
                   const Arguments&            arg,
                   uint32_t                    splitK,
                   uint32_t                    wgm,
                   double                      gpu_us,
                   double                      flush_us,
                   double                      gflops,
-                  double                      gpu_bytes = ArgumentLogging::NA_value,
-                  double                      cpu_us    = ArgumentLogging::NA_value,
-                  double                      norm      = ArgumentLogging::NA_value,
-                  double                      atol      = ArgumentLogging::NA_value,
-                  double                      rtol      = ArgumentLogging::NA_value)
+                  double                      gbytes = ArgumentLogging::NA_value,
+                  double                      cpu_us = ArgumentLogging::NA_value,
+                  double                      norm   = ArgumentLogging::NA_value,
+                  double                      atol   = ArgumentLogging::NA_value,
+                  double                      rtol   = ArgumentLogging::NA_value)
     {
         hipblaslt_internal_ostream name_list;
         hipblaslt_internal_ostream value_list;
@@ -187,10 +193,48 @@ public:
 
 #if __cplusplus >= 201703L
         // C++17
-        (ArgumentsHelper::apply<Args>(print, arg, T{}), ...);
+        //(ArgumentsHelper::apply<Args>(print, arg, T{}), ...);
+        switch(Tc)
+        {
+        case HIP_R_32F:
+            (ArgumentsHelper::apply<Args>(print, arg, float{}), ...);
+            break;
+        case HIP_R_64F:
+            (ArgumentsHelper::apply<Args>(print, arg, double{}), ...);
+            break;
+        case HIP_R_16F:
+            (ArgumentsHelper::apply<Args>(print, arg, hipblasLtHalf{}), ...);
+            break;
+        case HIP_R_32I:
+            (ArgumentsHelper::apply<Args>(print, arg, int32_t{}), ...);
+            break;
+        default:
+            hipblaslt_cerr << "Error type in log_args" << std::endl;
+            (ArgumentsHelper::apply<Args>(print, arg, float{}), ...);
+            break;
+        }
 #else
         // C++14. TODO: Remove when C++17 is used
-        (void)(int[]){(ArgumentsHelper::apply<Args>{}()(print, arg, T{}), 0)...};
+        //(void)(int[]){(ArgumentsHelper::apply<Args>{}()(print, arg, T{}), 0)...};
+        switch(Tc)
+        {
+        case HIP_R_32F:
+            (void)(int[]){(ArgumentsHelper::apply<Args>{}()(print, arg, float{}), 0)...};
+            break;
+        case HIP_R_64F:
+            (void)(int[]){(ArgumentsHelper::apply<Args>{}()(print, arg, double{}), 0)...};
+            break;
+        case HIP_R_16F:
+            (void)(int[]){(ArgumentsHelper::apply<Args>{}()(print, arg, hipblasLtHalf{}), 0)...};
+            break;
+        case HIP_R_32I:
+            (void)(int[]){(ArgumentsHelper::apply<Args>{}()(print, arg, int32_t{}), 0)...};
+            break;
+        default:
+            hipblaslt_cerr << "Error type in log_args" << std::endl;
+            (void)(int[]){(ArgumentsHelper::apply<Args>{}()(print, arg, float{}), 0)...};
+            break;
+        }
 #endif
 
         // Additional name and value list
@@ -206,23 +250,31 @@ public:
                      gpu_us,
                      flush_us,
                      gflops,
-                     gpu_bytes,
+                     gbytes,
                      cpu_us,
                      norm,
                      atol,
                      rtol);
 
-        if(solution_index > -1)
+        if(archName != "")
         {
-            str << name_list << "\n"
-                << value_list << "\n"
-                << "    --Solution index: " << solution_index << "\n"
+            auto delim = ",";
+            name_list << delim << "soulution_index";
+            value_list << delim << solution_index;
+
+            const char*   tuningEnv  = getenv("HIPBLASLT_TUNING_FILE");
+            std::string   tuningPath = tuningEnv;
+            std::ofstream file(tuningPath, std::ios::app);
+            file << value_list << delim << archName << delim << cuNum << std::endl;
+        }
+
+        str << name_list << "\n" << value_list << std::endl;
+
+        if(solution_name != "")
+        {
+            str << "    --Solution index: " << solution_index << "\n"
                 << "    --Solution name:  " << solution_name << "\n"
                 << "    --kernel name:    " << kernel_name << std::endl;
         }
-        else
-        {
-            str << name_list << "\n" << value_list << std::endl;
-        }
     }
 };
