@@ -26,6 +26,7 @@
 
 #pragma once
 
+#include "datatype_interface.hpp"
 #include "hipblaslt_datatype2string.hpp"
 #include "hipblaslt_math.hpp"
 #include "hipblaslt_ostream.hpp"
@@ -49,12 +50,6 @@ enum hipblaslt_argument : int;
 constexpr std::size_t MAX_SUPPORTED_NUM_PROBLEMS{32};
 struct Arguments
 {
-    enum ScalingFormat
-    {
-        None = 0,
-        Scalar,
-        Vector
-    };
     /*************************************************************************
      *                    Beginning Of Arguments                             *
      *************************************************************************/
@@ -133,23 +128,23 @@ struct Arguments
     float                     activation_arg1; // threshold when activation type is relu
     float                     activation_arg2; // upperbound when activation type is relu
 
-    hipDataType           bias_type;
-    hipblaslt_bias_source bias_source;
-    bool                  bias_vector;
-    ScalingFormat         scaleA;
-    ScalingFormat         scaleB;
-    bool                  scaleC;
-    bool                  scaleD;
-    bool                  scaleE;
-    bool                  scaleAlpha_vector;
-    bool                  amaxScaleA;
-    bool                  amaxScaleB;
-    bool                  amaxD;
-    bool                  c_equal_d;
-    bool                  HMM;
-    bool                  use_e;
-    bool                  gradient;
-    bool                  norm_check_assert;
+    hipDataType              bias_type;
+    hipblaslt_bias_source    bias_source;
+    bool                     bias_vector;
+    hipblaslt_scaling_format scaleA;
+    hipblaslt_scaling_format scaleB;
+    bool                     scaleC;
+    bool                     scaleD;
+    bool                     scaleE;
+    bool                     scaleAlpha_vector;
+    bool                     amaxScaleA;
+    bool                     amaxScaleB;
+    bool                     amaxD;
+    bool                     c_equal_d;
+    bool                     HMM;
+    bool                     use_e;
+    bool                     gradient;
+    bool                     norm_check_assert;
 
     // API related
     bool    use_ext;
@@ -159,7 +154,7 @@ struct Arguments
     bool    use_user_args;
     int32_t rotating;
     bool    use_gpu_timer;
-
+    float   skip_slow_solution_ratio;
     // tuning
     int32_t gsu_vector[MAX_SUPPORTED_NUM_PROBLEMS]; // This is for client
     int32_t wgm_vector[MAX_SUPPORTED_NUM_PROBLEMS]; // This is for client
@@ -256,6 +251,7 @@ struct Arguments
     OPER(use_user_args) SEP          \
     OPER(rotating) SEP               \
     OPER(use_gpu_timer) SEP          \
+    OPER(skip_slow_solution_ratio) SEP\
     OPER(gsu_vector) SEP             \
     OPER(wgm_vector) SEP             \
     OPER(print_solution_found) SEP   \
@@ -329,6 +325,171 @@ private:
     }
 };
 
+inline bool alpha_isnan_type(const Arguments& arg, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        return arg.alpha_isnan<float>();
+    case HIP_R_64F:
+        return arg.alpha_isnan<double>();
+    case HIP_R_16F:
+        return arg.alpha_isnan<hipblasLtHalf>();
+    case HIP_R_32I:
+        return arg.alpha_isnan<int32_t>();
+    default:
+        hipblaslt_cerr << "Error type in alpha_isnan_type()" << std::endl;
+        return arg.alpha_isnan<float>();
+    }
+}
+
+inline bool beta_isnan_type(const Arguments& arg, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        return arg.beta_isnan<float>();
+    case HIP_R_64F:
+        return arg.beta_isnan<double>();
+    case HIP_R_16F:
+        return arg.beta_isnan<hipblasLtHalf>();
+    case HIP_R_32I:
+        return arg.beta_isnan<int32_t>();
+    default:
+        hipblaslt_cerr << "Error type in beta_isnan_type()" << std::endl;
+        return arg.beta_isnan<float>();
+    }
+}
+
+inline void set_alpha_type(computeTypeInterface& h_alpha, const Arguments& arg, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        h_alpha.f32 = arg.get_alpha<float>();
+        return;
+    case HIP_R_64F:
+        h_alpha.f64 = arg.get_alpha<double>();
+        return;
+    case HIP_R_16F:
+        h_alpha.f16 = arg.get_alpha<hipblasLtHalf>();
+        return;
+    case HIP_R_32I:
+        h_alpha.i32 = arg.get_alpha<int32_t>();
+        return;
+    default:
+        hipblaslt_cerr << "Error type in set_alpha_type()" << std::endl;
+        return;
+    }
+}
+
+inline void set_beta_type(computeTypeInterface& h_beta, const Arguments& arg, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        h_beta.f32 = arg.get_beta<float>();
+        return;
+    case HIP_R_64F:
+        h_beta.f64 = arg.get_beta<double>();
+        return;
+    case HIP_R_16F:
+        h_beta.f16 = arg.get_beta<hipblasLtHalf>();
+        return;
+    case HIP_R_32I:
+        h_beta.i32 = arg.get_beta<int32_t>();
+        return;
+    default:
+        hipblaslt_cerr << "Error type in set_beta_type()" << std::endl;
+        return;
+    }
+}
+
+inline void set_computeInterface(computeTypeInterface& src, void* ptr, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        src.f32 = *(float*)ptr;
+        return;
+    case HIP_R_64F:
+        src.f64 = *(double*)ptr;
+        return;
+    case HIP_R_16F:
+        src.f16 = *(hipblasLtHalf*)ptr;
+        return;
+    case HIP_R_32I:
+        src.i32 = *(int32_t*)ptr;
+        return;
+    default:
+        hipblaslt_cerr << "Error type in set_computeInterface()" << std::endl;
+        return;
+    }
+}
+
+inline void set_computeInterface(computeTypeInterface& src, double value, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        src.f32 = static_cast<float>(value);
+        return;
+    case HIP_R_64F:
+        src.f64 = static_cast<double>(value);
+        return;
+    case HIP_R_16F:
+        src.f16 = static_cast<hipblasLtHalf>(value);
+        return;
+    case HIP_R_32I:
+        src.i32 = static_cast<int32_t>(value);
+        return;
+    default:
+        hipblaslt_cerr << "Error type in set_computeInterface()" << std::endl;
+        return;
+    }
+}
+
+inline void
+    mul_computeInterface(computeTypeInterface& dst, computeTypeInterface& src, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        dst.f32 *= src.f32;
+        return;
+    case HIP_R_64F:
+        dst.f64 *= src.f64;
+        return;
+    case HIP_R_16F:
+        dst.f16 *= src.f16;
+        return;
+    case HIP_R_32I:
+        dst.i32 *= src.i32;
+        return;
+    default:
+        hipblaslt_cerr << "Error type in mul_computeInterface()" << std::endl;
+        return;
+    }
+}
+
+inline double get_computeInterface(const computeTypeInterface src, hipDataType type)
+{
+    switch(type)
+    {
+    case HIP_R_32F:
+        return (double)src.f32;
+    case HIP_R_64F:
+        return (double)src.f64;
+    case HIP_R_16F:
+        return (double)src.f16;
+    case HIP_R_32I:
+        return (double)src.i32;
+    default:
+        hipblaslt_cerr << "Error type in get_computeInterface()" << std::endl;
+        return 0;
+    }
+}
+
 // We make sure that the Arguments struct is C-compatible
 static_assert(std::is_standard_layout<Arguments>{},
               "Arguments is not a standard layout type, and thus is "
