@@ -49,13 +49,13 @@ inline __host__ hip_bfloat16 float_to_bfloat16_truncate(float val)
 /*! \brief negate a value */
 
 template <class T>
-inline T negate(T x)
+inline __device__ __host__ T negate(T x)
 {
     return -x;
 }
 
 template <>
-inline hip_bfloat16 negate(hip_bfloat16 x)
+inline __device__ __host__ hip_bfloat16 negate(hip_bfloat16 x)
 {
 
 #if defined(__HIP_PLATFORM_AMD__)
@@ -69,14 +69,14 @@ inline hip_bfloat16 negate(hip_bfloat16 x)
 }
 
 template <>
-inline hipblaslt_f8_fnuz negate(hipblaslt_f8_fnuz x)
+inline __device__ __host__ hipblaslt_f8_fnuz negate(hipblaslt_f8_fnuz x)
 {
     x.data ^= 0x80;
     return x;
 }
 
 template <>
-inline hipblaslt_bf8_fnuz negate(hipblaslt_bf8_fnuz x)
+inline __device__ __host__ hipblaslt_bf8_fnuz negate(hipblaslt_bf8_fnuz x)
 {
     x.data ^= 0x80;
     return x;
@@ -84,14 +84,14 @@ inline hipblaslt_bf8_fnuz negate(hipblaslt_bf8_fnuz x)
 
 #ifdef ROCM_USE_FLOAT8
 template <>
-inline hipblaslt_f8 negate(hipblaslt_f8 x)
+inline __device__ __host__ hipblaslt_f8 negate(hipblaslt_f8 x)
 {
     x.data ^= 0x80;
     return x;
 }
 
 template <>
-inline hipblaslt_bf8 negate(hipblaslt_bf8 x)
+inline __device__ __host__ hipblaslt_bf8 negate(hipblaslt_bf8 x)
 {
     x.data ^= 0x80;
     return x;
