@@ -91,6 +91,23 @@
 #define TOO_MANY_DEVICES_STRING_GTEST "Succeeded\n" TOO_MANY_DEVICES_STRING
 #define HMM_NOT_SUPPORTED_GTEST "Succeeded\n" HMM_NOT_SUPPORTED
 
+inline bool is_bias_enabled(hipblasLtEpilogue_t value_)
+{
+    switch(value_)
+    {
+    case HIPBLASLT_EPILOGUE_BIAS:
+    case HIPBLASLT_EPILOGUE_GELU_BIAS:
+    case HIPBLASLT_EPILOGUE_RELU_BIAS:
+    case HIPBLASLT_EPILOGUE_GELU_AUX_BIAS:
+    case HIPBLASLT_EPILOGUE_DGELU_BGRAD:
+    case HIPBLASLT_EPILOGUE_BGRADA:
+    case HIPBLASLT_EPILOGUE_BGRADB:
+        return true;
+    default:
+        return false;
+    }
+};
+
 enum class hipblaslt_batch_type
 {
     none = 0,
@@ -194,7 +211,9 @@ public:
     hipblaslt_local_matmul_descr(hipblasOperation_t   opA,
                                  hipblasOperation_t   opB,
                                  hipblasComputeType_t compute_type,
-                                 hipDataType          scale_type)
+                                 hipDataType          scale_type,
+                                 hipDataType compute_input_typeA = HIPBLASLT_DATATYPE_INVALID,
+                                 hipDataType compute_input_typeB = HIPBLASLT_DATATYPE_INVALID)
     {
         this->m_status = hipblasLtMatmulDescCreate(&this->m_descr, compute_type, scale_type);
 
@@ -202,9 +221,22 @@ public:
             this->m_descr, HIPBLASLT_MATMUL_DESC_TRANSA, &opA, sizeof(int32_t));
         hipblasLtMatmulDescSetAttribute(
             this->m_descr, HIPBLASLT_MATMUL_DESC_TRANSB, &opB, sizeof(int32_t));
+
+        hipblasLtMatmulDescSetAttribute(this->m_descr,
+                                        HIPBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT,
+                                        &compute_input_typeA,
+                                        sizeof(void*));
+        hipblasLtMatmulDescSetAttribute(this->m_descr,
+                                        HIPBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_B_EXT,
+                                        &compute_input_typeB,
+                                        sizeof(void*));
     }
 
-    ~hipblaslt_local_matmul_descr() {}
+    ~hipblaslt_local_matmul_descr()
+    {
+        if(this->m_status == HIPBLAS_STATUS_SUCCESS)
+            hipblasLtMatmulDescDestroy(this->m_descr);
+    }
 
     hipblaslt_local_matmul_descr(const hipblaslt_local_matmul_descr&)            = delete;
     hipblaslt_local_matmul_descr(hipblaslt_local_matmul_descr&&)                 = delete;
@@ -454,4 +486,41 @@ void print_strided_batched(
     hipblaslt_cout << std::flush;
 }
 
+/* ===================================================================== */
+/*! \brief For special numerical types, to convert a value to such type. */
+template <typename T, typename Accumulator>
+typename std::enable_if<std::is_same<int8_t, T>::value, T>::type saturate_cast(Accumulator val)
+{
+    if constexpr(std::is_same<Accumulator, hipblasLtHalf>::value
+                 || std::is_same<Accumulator, hip_bfloat16>::value)
+    {
+        float tmp = std::nearbyint((float)val); //round to even
+        if(tmp > static_cast<float>(127))
+            tmp = static_cast<float>(127);
+        else if(tmp < static_cast<float>(-128))
+            tmp = static_cast<float>(-128);
+        return static_cast<T>(tmp);
+    }
+    else
+    {
+        if constexpr(std::is_same<Accumulator, float>::value
+                     || std::is_same<Accumulator, double>::value)
+            val = std::nearbyint(val); //round to even
+        if(val > static_cast<Accumulator>(127))
+            val = static_cast<Accumulator>(127);
+        else if(val < static_cast<Accumulator>(-128))
+            val = static_cast<Accumulator>(-128);
+        return static_cast<T>(val);
+    }
+}
+
+/* ==================================================================== */
+/*! \brief For common numerical types, to convert a value to such type. */
+template <typename T, typename Accumulator>
+typename std::enable_if<!std::is_same<int8_t, T>::value, T>::type saturate_cast(Accumulator val)
+{
+    return static_cast<T>(val);
+}
+
 std::vector<void*> benchmark_allocation();
+int32_t            hipblaslt_get_arch_major();
