@@ -27,6 +27,7 @@
 #pragma once
 #ifndef ROCBLASLT_UTILS_HPP
 #define ROCBLASLT_UTILS_HPP
+#include "auxiliary.hpp"
 #include "handle.h"
 #include "utility.hpp"
 
@@ -43,12 +44,12 @@ inline rocblaslt_status getOriginalSizes(hipblasOperation_t opA,
     // values of num_* are values after been transposed, redirect to before which
     // been transposed. initialized m,n,k by NN.
     m = num_rows_a, n = num_cols_b, k = num_cols_a;
-    if(opA == HIPBLAS_OP_T)
+    if(opA == HIPBLAS_OP_T || opA == HIPBLAS_OP_C)
     {
         m = num_cols_a;
         k = num_rows_a;
     }
-    if(opB == HIPBLAS_OP_T)
+    if(opB == HIPBLAS_OP_T || opB == HIPBLAS_OP_C)
     {
         n = num_rows_b;
         if(k != num_cols_b)
@@ -148,6 +149,13 @@ inline rocblaslt_status validateMatmulArgs(int64_t                       m,
                                            const void*                   beta,
                                            const void*                   c,
                                            const void*                   d,
+                                           hipDataType                   type_a,
+                                           hipDataType                   type_b,
+                                           hipDataType                   type_c,
+                                           hipDataType                   type_d,
+                                           rocblaslt_compute_type        compute_type,
+                                           hipblasOperation_t            opA,
+                                           hipblasOperation_t            opB,
                                            int                           num_batches_a  = 1,
                                            int                           num_batches_b  = 1,
                                            int                           num_batches_c  = 1,
@@ -159,6 +167,56 @@ inline rocblaslt_status validateMatmulArgs(int64_t                       m,
                                            const rocblaslt_pointer_mode& pointermode
                                            = rocblaslt_pointer_mode_host)
 {
+    rocblaslt_status status = rocblaslt_status_continue;
+
+    if(!(type_a == HIP_R_32F && type_b == HIP_R_32F && type_c == HIP_R_32F && type_d == HIP_R_32F)
+       && compute_type == rocblaslt_compute_f32_fast_xf32)
+        status = rocblaslt_status_not_implemented;
+    if(!((type_a == HIP_R_8I && type_b == HIP_R_8I && type_c == HIP_R_32I && type_d == HIP_R_32I)
+         || (type_a == HIP_R_8I && type_b == HIP_R_8I && type_c == HIP_R_8I && type_d == HIP_R_8I))
+       && compute_type == rocblaslt_compute_i32)
+        status = rocblaslt_status_not_implemented;
+    if(!strcmp(hip_datatype_to_string(type_a), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
+       || !strcmp(hip_datatype_to_string(type_b), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
+       || !strcmp(hip_datatype_to_string(type_c), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
+       || !strcmp(hip_datatype_to_string(type_d), hip_datatype_to_string(HIPBLASLT_DATATYPE_INVALID))
+       || !strcmp(rocblaslt_compute_type_string(compute_type),
+                  rocblaslt_compute_type_string(ROCBLASLT_COMPUTE_TYPE_INVALID)))
+        status = rocblaslt_status_not_implemented;
+
+    if(status != rocblaslt_status_continue)
+    {
+        log_error(__func__,
+                  "invalid args",
+                  "datatype",
+                  "matA",
+                  hipDataType_to_string(type_a),
+                  "matB",
+                  hipDataType_to_string(type_b),
+                  "matC",
+                  hipDataType_to_string(type_c),
+                  "matD",
+                  hipDataType_to_string(type_d),
+                  "computeType",
+                  rocblaslt_compute_type_string(compute_type));
+        return status;
+    }
+
+    if(opA == HIPBLASLT_OPERATION_INVALID || opB == HIPBLASLT_OPERATION_INVALID)
+        status = rocblaslt_status_not_implemented;
+
+    if(status != rocblaslt_status_continue)
+    {
+        log_error(__func__,
+                  "invalid args",
+                  "op",
+                  "opA",
+                  hipblasOperation_to_string(opA),
+                  "opB",
+                  hipblasOperation_to_string(opB));
+        return status;
+    }
+
     // sizes must not be negative
     if(batch_stride_a < 0 || batch_stride_b < 0 || batch_stride_c < 0 || batch_stride_d < 0)
     {
@@ -326,61 +384,62 @@ inline rocblaslt_status rocblaslt_matmul_valid_args(const rocblaslt_matmul_desc
     n = num_cols_d;
     k = (opA == HIPBLAS_OP_N) ? num_cols_a : num_rows_a;
 
-    auto status = validateMatmulArgs(m,
-                                     n,
-                                     k,
-                                     alpha,
-                                     A,
-                                     B,
-                                     beta,
-                                     C,
-                                     D,
-                                     num_batches_a,
-                                     num_batches_b,
-                                     num_batches_c,
-                                     num_batches_d,
-                                     batch_stride_a,
-                                     batch_stride_b,
-                                     batch_stride_c,
-                                     batch_stride_d, 
-                                     matmul_descr->pointermode);
+    auto matmul_status = validateMatmulArgs(m,
+                                            n,
+                                            k,
+                                            alpha,
+                                            A,
+                                            B,
+                                            beta,
+                                            C,
+                                            D,
+                                            matA->type,
+                                            matB->type,
+                                            matC->type,
+                                            matD->type,
+                                            compute_type,
+                                            matmul_descr->op_A,
+                                            matmul_descr->op_B,
+                                            num_batches_a,
+                                            num_batches_b,
+                                            num_batches_c,
+                                            num_batches_d,
+                                            batch_stride_a,
+                                            batch_stride_b,
+                                            batch_stride_c,
+                                            batch_stride_d,
+                                            matmul_descr->pointermode);
 
-    if(status == rocblaslt_status_continue)
-    {
-        if(!(matA->type == HIP_R_32F && matB->type == HIP_R_32F && matC->type == HIP_R_32F
-             && matD->type == HIP_R_32F)
-           && compute_type == rocblaslt_compute_f32_fast_xf32)
-            status = rocblaslt_status_not_implemented;
-        if(!((matA->type == HIP_R_8I && matB->type == HIP_R_8I && matC->type == HIP_R_32I
-              && matD->type == HIP_R_32I)
-             || (matA->type == HIP_R_8I && matB->type == HIP_R_8I && matC->type == HIP_R_8I
-                 && matD->type == HIP_R_8I))
-           && compute_type == rocblaslt_compute_i32)
-            status = rocblaslt_status_not_implemented;
-    }
-    const void* alphaVecPtr = matmul_descr->pointermode ? alpha : nullptr;
-    if(status == rocblaslt_status_continue)
-        status = rocblaslt_epilogue_valid_args(matmul_descr->epilogue,
-                                               num_rows_d,
-                                               num_cols_d,
-                                               matD->type,
-                                               matmul_descr->bias_type,
-                                               matmul_descr->e,
-                                               matmul_descr->lde,
-                                               matmul_descr->stride_e,
-                                               matmul_descr->bias,
-                                               alphaVecPtr,
-                                               alpha,
-                                               matmul_descr->isScaleAVec,
-                                               matmul_descr->isScaleBVec,
-                                               E,
-                                               lde,
-                                               batch_stride_e,
-                                               bias,
-                                               bias_type,
-                                               scaleAlphaVec,
-                                               gradient);
-    return status;
+    const void* alphaVecPtr     = matmul_descr->pointermode ? alpha : nullptr;
+    auto        epilogue_status = rocblaslt_epilogue_valid_args(matmul_descr->epilogue,
+                                                         num_rows_d,
+                                                         num_cols_d,
+                                                         matD->type,
+                                                         matmul_descr->bias_type,
+                                                         matmul_descr->e,
+                                                         matmul_descr->lde,
+                                                         matmul_descr->stride_e,
+                                                         matmul_descr->bias,
+                                                         alphaVecPtr,
+                                                         alpha,
+                                                         matmul_descr->isScaleAVec,
+                                                         matmul_descr->isScaleBVec,
+                                                         E,
+                                                         lde,
+                                                         batch_stride_e,
+                                                         bias,
+                                                         bias_type,
+                                                         scaleAlphaVec,
+                                                         gradient);
+
+    // rocblaslt_epilogue_valid_args must to be called otherwise bias_type will be garbage value
+    if(matmul_status != rocblaslt_status_continue)
+        return matmul_status;
+
+    if(epilogue_status != rocblaslt_status_continue)
+        return epilogue_status;
+
+    return rocblaslt_status_continue;
 }
 
 // Assign 1 to onePtr then set set the address to dst.
