@@ -33,9 +33,12 @@
 #include "logging.h"
 #include <algorithm>
 #include <exception>
+#include <mutex>
 
 #pragma STDC CX_LIMITED_RANGE ON
 
+static std::mutex log_mutex;
+
 inline bool isAligned(const void* pointer, size_t byte_count)
 {
     return reinterpret_cast<uintptr_t>(pointer) % byte_count == 0;
@@ -61,17 +64,38 @@ constexpr const char* rocblaslt_datatype_string(hipDataType type)
     }
 }
 
-// return precision string for rocblaslt_compute_type
+bool rocblaslt_is_complex_datatype(hipDataType type);
+
 constexpr const char* rocblaslt_compute_type_string(rocblaslt_compute_type type)
 {
     switch(type)
     {
+    case rocblaslt_compute_f16:
+        return "f16_r";
     case rocblaslt_compute_f32:
-        return "f32";
+        return "f32_r";
     case rocblaslt_compute_f32_fast_xf32:
-        return "xf32";
+        return "xf32_r";
     case rocblaslt_compute_i32:
-        return "i32";
+        return "i32_r";
+    case rocblaslt_compute_f64:
+        return "f64_r";
+    case rocblaslt_compute_f32_fast_f16:
+        return "f32_f16_r";
+    case rocblaslt_compute_f32_fast_bf16:
+        return "f32_bf16_r";
+    case rocblaslt_compute_f32_fast_f8_ocp:
+    case rocblaslt_compute_f32_fast_f8_fnuz:
+        return "f32_f8_r";
+    case rocblaslt_compute_f32_fast_bf8_ocp:
+    case rocblaslt_compute_f32_fast_bf8_fnuz:
+        return "f32_bf8_r";
+    case rocblaslt_compute_f32_fast_f8bf8_ocp:
+    case rocblaslt_compute_f32_fast_f8bf8_fnuz:
+        return "f32_f8bf8_r";
+    case rocblaslt_compute_f32_fast_bf8f8_ocp:
+    case rocblaslt_compute_f32_fast_bf8f8_fnuz:
+        return "f32_bf8f8_r";
     default:
         return "invalidType";
     }
@@ -85,6 +109,8 @@ constexpr const char* rocblaslt_transpose_letter(hipblasOperation_t op)
         return "N";
     case HIPBLAS_OP_T:
         return "T";
+    case HIPBLAS_OP_C:
+        return "C";
     default:
         return "invalidTranspose";
     }
@@ -179,6 +205,7 @@ void log_base(rocblaslt_layer_mode layer_mode, const char* func, H head, Ts&&...
 {
     if(get_logger_layer_mode() & layer_mode)
     {
+        std::lock_guard<std::mutex> lock(log_mutex);
         std::string comma_separator = " ";
 
         std::ostream* os = get_logger_os();
@@ -247,11 +274,50 @@ void log_api(const char* func, H head, Ts&&... xs)
 template <typename... Ts>
 void log_bench(const char* func, Ts&&... xs)
 {
+    std::lock_guard<std::mutex> lock(log_mutex);
     std::ostream* os = get_logger_os();
     *os << "hipblaslt-bench ";
     log_arguments_bench(*os, std::forward<Ts>(xs)...);
     *os << std::endl;
 }
+
+inline void log_bench_from_str(std::string s)
+{
+    std::lock_guard<std::mutex> lock(log_mutex);
+    std::ostream*               os = get_logger_os();
+    *os << s.c_str();
+    *os << std::endl;
+}
+
+template <typename... Ts>
+inline std::string log_str(const char* func, Ts&&... xs)
+{
+    std::stringstream ss;
+    ss << "hipblaslt-bench ";
+    log_arguments_bench(ss, std::forward<Ts>(xs)...);
+    return ss.str();
+}
+
+// if profile logging is turned on with
+// (handle->layer_mode & rocblaslt_layer_mode_log_profile) == true
+// log_profile will call argument_profile to profile actual arguments,
+// keeping count of the number of times each set of arguments is used
+template <typename... Ts>
+void log_profile(const char* func, Ts&&... xs)
+{
+    // Make a tuple with the arguments
+    auto tup = std::make_tuple("function", func, std::forward<Ts>(xs)...);
+
+    // Set up profile
+    static argument_profile<decltype(tup)> profile(get_logger_os());
+
+    // Add at_quick_exit handler in case the program exits early
+    static int aqe = at_quick_exit([] { profile.~argument_profile(); });
+
+    // Profile the tuple
+    profile(std::move(tup));
+}
+
 // Convert the current C++ exception to rocblaslt_status
 // This allows extern "C" functions to return this function in a catch(...)
 // block while converting all C++ exceptions to an equivalent rocblaslt_status
