@@ -98,8 +98,9 @@ constexpr const char* hipblas_operation_to_string(hipblasOperation_t value)
         return "T";
     case HIPBLAS_OP_C:
         return "C";
+    default:
+        return "invalid";
     }
-    return "invalid";
 }
 
 HIPBLASLT_EXPORT
@@ -117,7 +118,7 @@ constexpr hipblasOperation_t char_to_hipblas_operation(char value)
     case 'c':
         return HIPBLAS_OP_C;
     default:
-        return static_cast<hipblasOperation_t>(0);
+        return HIPBLASLT_OPERATION_INVALID;
     }
 }
 
@@ -161,6 +162,8 @@ constexpr const char* hipblas_computetype_to_string(hipblasComputeType_t type)
 {
     switch(type)
     {
+    case HIPBLAS_COMPUTE_16F:
+        return "f16_r";
     case HIPBLAS_COMPUTE_32F:
         return "f32_r";
     case HIPBLAS_COMPUTE_32F_FAST_TF32:
@@ -214,7 +217,7 @@ HIPBLASLT_EXPORT
 constexpr hipDataType string_to_hip_datatype_assert(const std::string& value)
 {
     auto datatype = string_to_hip_datatype(value);
-    if(static_cast<int>(datatype) == 0)
+    if(datatype == HIPBLASLT_DATATYPE_INVALID)
     {
         std::cout << "The supported types are f32_r, f64_r, f16_r, bf16_r, f8_r, bf8_r, i8_r, i32_r." << std::endl;
         exit(1);
@@ -232,14 +235,14 @@ constexpr hipblasComputeType_t string_to_hipblas_computetype(const std::string&
         value == "i32_r" || value == "i" ? HIPBLAS_COMPUTE_32I :
         value == "f32_f16_r" ? HIPBLAS_COMPUTE_32F_FAST_16F :
         value == "f32_bf16_r" ? HIPBLAS_COMPUTE_32F_FAST_16BF :
-        static_cast<hipblasComputeType_t>(0);
+        HIPBLASLT_COMPUTE_TYPE_INVALID;
 }
 
 HIPBLASLT_EXPORT
 constexpr hipblasComputeType_t string_to_hipblas_computetype_assert(const std::string& value)
 {
     auto computetytpe = string_to_hipblas_computetype(value);
-    if(static_cast<int>(computetytpe) == 0)
+    if(computetytpe == HIPBLASLT_COMPUTE_TYPE_INVALID)
     {
         std::cout << "The supported types are f32_r, xf32_r, f64_r, i32_r, f32_f16_r." << std::endl;
         exit(1);
