@@ -20,10 +20,32 @@
 # CTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ################################################################################
 
+from .TensileInstructions import DataType
 from .AsmAddressCalculation import AddrCalculation
 from .Utils import DataDirection
 
 from math import ceil, trunc, modf
+from dataclasses import dataclass, field
+from typing import Optional
+
+@dataclass
+class VectorUnit:
+  dataType: Optional[DataType] = None
+  dstVgpr: int    = -1
+  offsetVgpr: int = -1
+  turn: int       = 0
+  ldsOffset: int  = 0
+
+@dataclass
+class VectorDataTypes:
+  scaleA: VectorUnit     = field(default_factory=VectorUnit)
+  scaleB: VectorUnit     = field(default_factory=VectorUnit)
+  bias: VectorUnit       = field(default_factory=VectorUnit)
+  scaleAlpha: VectorUnit = field(default_factory=VectorUnit)
+
+  def isValid(self):
+    return self.scaleA.dataType or self.scaleB.dataType or self.bias.dataType \
+    or self.scaleAlpha.dataType
 
 ##############################################################################
 # StoreState
@@ -39,7 +61,7 @@ class StoreState:
     # the generation of the store code.
     ##############################################################################
     class StoreConstConfig:
-        def __init__(self, kernelWriter, kernel, ss, gwvw, edge, beta, atomic):
+        def __init__(self, kernelWriter, kernel, ss, gwvw, edge, beta, atomic, isWorkspace=False):
             self.gwvw = gwvw
             self.lsu = kernel["LocalSplitU"]
 
@@ -64,7 +86,7 @@ class StoreState:
                 self.numTempSgprPerBatch   = 2 * kernelWriter.states.laneSGPRCount
 
             if self.numMaskSgprPerElement:
-                numSgprAvailable = kernelWriter.consts.maxSgprs - kernelWriter.sgprPool.size() + kernelWriter.sgprPool.availableBlockAtEnd()
+                numSgprAvailable = kernelWriter.states.regCaps["MaxSgpr"] - kernelWriter.sgprPool.size() + kernelWriter.sgprPool.availableBlockAtEnd()
                 numSgprAvailable = numSgprAvailable & ~0x1 # make sure it's aligned
                 #print("numSgprAvailable=", numSgprAvailable)
                 self.numElementsPerBatchLimitedBySgprs = (numSgprAvailable - self.numTempSgprPerBatch - self.numMaskSgprPerBatch) // self.numMaskSgprPerElement
@@ -76,14 +98,18 @@ class StoreState:
                 self.numElementsPerBatchLimitedBySgprs = 1 # dummy value
                   #assert self.numElementsPerBatchLimitedBySgprs > 0, "numElementsPerBatchLimitedBySgprs=0 for %s"%self.kernelName
 
+            bpeC = kernelWriter.states.bpeCexternal
+            if isWorkspace:
+                bpeC = kernelWriter.states.bpeCinternal
+
             if atomic:
                 # flat atomics have another VGPR to allow different data for return#
                 regsPerElement = 2 if kernel["BufferStore"] else (3 + 1) # + 1 for alignment
                 # The atomic loop processes multiple elements in single instruction
                 # so will use VGPR from consec elements? TODO
-                self.numVgprsPerDataPerVI = (1.0 * regsPerElement * kernelWriter.states.bpeCexternal) / kernelWriter.states.bpr
+                self.numVgprsPerDataPerVI = (1.0 * regsPerElement * bpeC) / kernelWriter.states.bpr
             elif beta:
-                self.numVgprsPerDataPerVI = (1.0 * kernelWriter.states.bpeCexternal) / kernelWriter.states.bpr
+                self.numVgprsPerDataPerVI = (1.0 * bpeC) / kernelWriter.states.bpr
             else:
                 self.numVgprsPerDataPerVI = 0.0
 
@@ -98,17 +124,19 @@ class StoreState:
             # Really only used if gwvw=1 - edge cases
             # exception: data vgpr cannot be shared if UseInitialStridesCD is enabled and card enable EccHalf,
             #            since each buffer_load_short would overwrite undefined 16bit as zero.
-            self.halfDataRegPerVI = gwvw*self.numVgprsPerDataPerVI == 0.5 and not (kernel["ProblemType"]["UseInitialStridesCD"] and kernelWriter.states.archCaps["HasEccHalf"]) and not (kernel["ProblemType"]["DestDataType"].numRegisters() == 0.25)
+            self.halfDataRegPerVI = gwvw*self.numVgprsPerDataPerVI == 0.5 and not (kernel["ProblemType"]["UseInitialStridesCD"] and (kernelWriter.states.archCaps["HasEccHalf"] or not kernelWriter.states.asmCaps["HasWMMA_V1"])) and not (kernel["ProblemType"]["DestDataType"].numRegisters() == 0.25)
             # indicates the VGPRs index offset from LSU Reduction.
             # Used for multi-batch/Edge cases.
             self.lsuStartVgprOffset = 0
 
     # StoreState constructor:
-    def __init__(self, kernelWriter, kernel, gwvw, edge, beta, atomic, elements):
+    def __init__(self, kernelWriter, kernel, gwvw, edge, beta, atomic, elements, vectorDataTypes, dim, isWorkspace=False):
         self.kernelWriter = kernelWriter
         self.kernel = kernel
         self.lsu = kernel["LocalSplitU"]
         self.lsuStartVgprOffset = 0
+        self.factorDim = dim
+        self.vectorDataTypes = vectorDataTypes
 
         self.isReset = False
         #--
@@ -158,7 +186,7 @@ class StoreState:
             if not atomic and len(kernel["PackedC1IndicesX"]) == 1:
                 self.optSrdIncForRow = 1
 
-        if kernel["StoreRemapVectorWidth"]:
+        if kernel["StoreRemapVectorWidth"] and not isWorkspace:
             self.optSrdIncForRow = 1
 
         if kernel["ProblemType"]["UseInitialStridesCD"]:
@@ -173,7 +201,7 @@ class StoreState:
         assert (not (self.optSingleColVgpr and self.optSharedColVgpr))
 
 
-        self.cfg = self.StoreConstConfig(kernelWriter, kernel, self, gwvw, edge, beta, atomic)
+        self.cfg = self.StoreConstConfig(kernelWriter, kernel, self, gwvw, edge, beta, atomic, isWorkspace)
 
         # Use to detect new rows:
         self.lastCoordOffset1 = 0
@@ -183,6 +211,14 @@ class StoreState:
 
         # epilogue related
         self.useBias = kernelWriter.states.useBias
+        self.referenceVgprDim = [[], []]
+        if self.useBias == DataDirection.READ:
+            self.referenceVgprDim[self.factorDim].append("Bias")
+        if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+            self.referenceVgprDim[self.factorDim].append("ScaleAlpha")
+        if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+            self.referenceVgprDim[0].append("ScaleA")
+            self.referenceVgprDim[1].append("ScaleB")
 
         if self.optSharedColVgpr:
             numCols = len([e for e in elements if e[0] == 0 and e[2] == 0]) # count #elements with row d1=v1==0
@@ -204,16 +240,25 @@ class StoreState:
                 self.sharedColEVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColEVgprs for packed elements")
             else:
                 self.sharedColEVgprs = None
+            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                if self.referenceVgprDim[self.factorDim] and self.referenceVgprDim[self.factorDim][0] == "ScaleAlpha":
+                    self.sharedColScaleAlphaVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleAlphaVecVgprs for packed elements")
+                else:
+                    self.sharedColScaleAlphaVecVgprs = None
+            else:
+                self.sharedColScaleAlphaVecVgprs = None
             if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                self.sharedColScaleAVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleAVecVgprs for packed elements")
-                self.sharedColScaleBVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleBVecVgprs for packed elements")
+                if self.referenceVgprDim[0] and self.referenceVgprDim[0][0] == "ScaleA":
+                    self.sharedColScaleAVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleAVecVgprs for packed elements")
+                else:
+                    self.sharedColScaleAVecVgprs = None
+                if self.referenceVgprDim[1] and self.referenceVgprDim[1][0] == "ScaleB":
+                    self.sharedColScaleBVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleBVecVgprs for packed elements")
+                else:
+                    self.sharedColScaleBVecVgprs = None
             else:
                 self.sharedColScaleAVecVgprs = None
                 self.sharedColScaleBVecVgprs = None
-            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                self.sharedColScaleAlphaVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleAlphaVecVgprs for packed elements")
-            else:
-                self.sharedColScaleAlphaVecVgprs = None
         elif self.optSingleColVgpr:
             self.numAddrVgpr = 1
             self.sharedColDVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColDVgprs")
@@ -238,16 +283,25 @@ class StoreState:
                 self.sharedColEVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColEVgprs for packed elements")
             else:
                 self.sharedColEVgprs = None
+            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                if self.referenceVgprDim[self.factorDim] and self.referenceVgprDim[self.factorDim][0] == "ScaleAlpha":
+                    self.sharedColScaleAlphaVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleAlphaVecVgprs for packed elements")
+                else:
+                    self.sharedColScaleAlphaVecVgprs = None
+            else:
+                self.sharedColScaleAlphaVecVgprs = None
             if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                self.sharedColScaleAVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleAVecVgprs for packed elements")
-                self.sharedColScaleBVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleBVecVgprs for packed elements")
+                if self.referenceVgprDim[0] and self.referenceVgprDim[0][0] == "ScaleA":
+                    self.sharedColScaleAVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleAVecVgprs for packed elements")
+                else:
+                    self.sharedColScaleAVecVgprs = None
+                if self.referenceVgprDim[1] and self.referenceVgprDim[1][0] == "ScaleB":
+                    self.sharedColScaleBVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleBVecVgprs for packed elements")
+                else:
+                    self.sharedColScaleBVecVgprs = None
             else:
                 self.sharedColScaleAVecVgprs = None
                 self.sharedColScaleBVecVgprs = None
-            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                self.sharedColScaleAlphaVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleAlphaVecVgprs for packed elements")
-            else:
-                self.sharedColScaleAlphaVecVgprs = None
         else:
             self.numAddrVgpr = 0
             self.sharedColEVgprs    = None
@@ -262,9 +316,22 @@ class StoreState:
         # For detecting when we are running first batch
         self.firstBatch = True
 
+        # TODO code migrated from globalWrite, comments outdated with respect to features like activation
+        # how many vgprs are needed for zero elements
+        # 2 for addressC in vgpr for addition - already checked out
+        # 2 for coord0,1 of thread - already checked out
+        # 2 for tmp - already checked out
+
+        # 5 = how many vgprs are needed per element (flat)
+        #  - 2 for addr
+        #  - 3 for GLOBAL_OFFSET_C calculation (can overlap below, therefore max)
+        #  - if beta gwvw*rpe for new value
+        #  - if atomic 2*rpe for old and cmp values
+
         # Calculate numVgprsPerElement
         # print("numVgprsPerAddr=%u, numVgprsPerDataPerVI=%u, numVgprPerValuC=%u"%(self.cfg.numVgprsPerAddr, self.cfg.numVgprsPerDataPerVI, self.cfg.numVgprPerValuC))
         self.numVgprsPerElement = self.cfg.numVgprPerValuC*gwvw + self.cfg.numVgprsPerAddr + int(ceil(self.cfg.numVgprsPerDataPerVI * gwvw))
+        # TODO STREAM-K Update numVgprsPerElement for workspace
         if kernel["GroupLoadStore"] and kernel["ProblemType"]["UseBeta"]:
             self.numVgprsPerElement += self.cfg.numVgprsPerAddr
         if kernel["ProblemType"]["UseE"] and (kernel["GlobalSplitU"] == 1):
@@ -273,21 +340,28 @@ class StoreState:
             if (kernel["ProblemType"]["Gradient"] and kernel["ProblemType"]["ActivationType"] != 'none'):
                 numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
                 self.numVgprsPerElement += numVgprs * gwvw # Loaded data
+        # We will use the same vgpr + ds_offset to load the vec addr
+
         if self.useBias != DataDirection.NONE:
             self.numVgprsPerElement += self.cfg.numVgprsPerAddr  # Bias address
             if self.useBias == DataDirection.READ:
                 numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                self.numVgprsPerElement += numVgprs * gwvw  # Loaded data
-
-        if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-            self.numVgprsPerElement += self.cfg.numVgprsPerAddr * 2  # ScaleAVec + ScaleBVec address
-            numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-            self.numVgprsPerElement += numVgprs * gwvw + (numVgprs * (2 if gwvw >= 2 else 1)) # Loaded data
+                self.numVgprsPerElement += numVgprs * gwvw if self.factorDim == 0 else min(gwvw, 2) # Loaded data
 
         if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-            self.numVgprsPerElement += self.cfg.numVgprsPerAddr  # ScaleAlphaVec address
+            if self.referenceVgprDim[self.factorDim] and self.referenceVgprDim[self.factorDim][0] == "ScaleAlpha":
+                self.numVgprsPerElement += self.cfg.numVgprsPerAddr  # ScaleAlphaVec address
             numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-            self.numVgprsPerElement += numVgprs * gwvw  # Loaded data
+            self.numVgprsPerElement += numVgprs * gwvw if self.factorDim == 0 else min(gwvw, 2) # Loaded data
+
+        if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+            if self.referenceVgprDim[0] and self.referenceVgprDim[0][0] == "ScaleA":
+                self.numVgprsPerElement += self.cfg.numVgprsPerAddr # ScaleAVec address
+            if self.referenceVgprDim[1] and self.referenceVgprDim[1][0] == "ScaleB":
+                self.numVgprsPerElement += self.cfg.numVgprsPerAddr # ScaleBVec address
+            numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+            self.numVgprsPerElement += numVgprs * gwvw + (numVgprs * min(gwvw, 2)) # Loaded data
+
         # Calculate align
         self.align = 1
         # align adjustment
@@ -377,7 +451,7 @@ class StoreState:
 
         return self.elementCoord0, self.elementCoord1
 
-    def setupStoreElementsForBatch(self, kernel, gwvw, batchElements, batchElementSgprs, isOptNLL, factorDim):
+    def setupStoreElementsForBatch(self, kernel, gwvw, batchElements, batchElementSgprs, isOptNLL, factorDim, isWorkspace=False):
 
         self.elementAddr              = []
         self.elementDataE             = []
@@ -394,6 +468,9 @@ class StoreState:
         #generate elementCoord0/1
         self.getStoreElementsInfoForBatch(kernel, batchElements)
         kw = self.kernelWriter
+        dataType = kernel["ProblemType"]["DestDataType"]
+        if isWorkspace:
+            dataType = kernel["ProblemType"]["ComputeDataType"]
 
         biasVgprMap = {}
         scaleAVecVgprMap = {}
@@ -421,6 +498,122 @@ class StoreState:
 
             (d1,d0,vc1,vc0) = element
 
+            # if numVgprsPerDataPerVI == 0.5, then two consecutive elements
+            # should have same data pointer, next should move.
+            if self.cfg.numVgprsPerDataPerVI > 0:
+                if self.cfg.halfDataRegPerVI:
+                    # TODO- check (H,H,H,H,S,S)
+                    if kernel["ProblemType"]["HighPrecisionAccumulate"] and \
+                       (dataType.isBFloat16() or dataType.isHalf()):
+                        data = kw.vgprPool.checkOutAligned(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
+                              int(ceil(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw))), "writeBatch-data for ei=%u and ei=%u"%(elementIdx,elementIdx+1), preventOverflow=not isOptNLL)
+                    else:
+                        if elementIdx%2 == 0:
+                            # allocate for two elements:
+                            data = kw.vgprPool.checkOutAligned(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
+                                   int(ceil(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw))), "writeBatch-data for ei=%u and ei=%u"%(elementIdx,elementIdx+1), preventOverflow=not isOptNLL)
+                            lastData = data
+                        else:
+                            data = lastData
+                            del lastData
+                else:
+                    if self.cfg.numVgprsPerDataPerVI == 0.5 or self.cfg.numVgprsPerDataPerVI == 0.25:
+                        data = kw.vgprPool.checkOutAligned(int(ceil(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw)), \
+                              int(ceil(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw)), "writeBatch-data for ei=%u"%elementIdx, preventOverflow=False)
+                    else:
+                        data = kw.vgprPool.checkOutAligned(int(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
+                              int(ceil(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw)), "writeBatch-data for ei=%u"%elementIdx, preventOverflow=False)
+                    #data = kw.vgprPool.checkOut(int(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
+                    #      "writeBatch-data for ei=%u"%elementIdx, preventOverflow=False)
+            else:
+                data = 0
+            self.elementData.append(data)
+
+            if self.useBias == DataDirection.READ:
+                coordOffset = coordOffset0 if factorDim == 0 else coordOffset1
+                if coordOffset in biasVgprMap:
+                    dataBias = biasVgprMap[coordOffset]
+                else:
+                    gwvw = self.cfg.gwvw if factorDim == 0 else min(self.cfg.gwvw, 2)
+                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+                    dataBias = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
+                                int(ceil(numVgprs*gwvw)), "bias data for ei=%u"%elementIdx, preventOverflow=False)
+                    biasVgprMap[coordOffset] = dataBias
+            else:
+                dataBias = 0
+            self.elementDataBias.append(dataBias)
+            # Only needed in gradient activation
+            if (kernel["ProblemType"]["Gradient"] and kernel["ProblemType"]["ActivationType"] != 'none' and kernel["ProblemType"]["UseE"]) and (kernel["GlobalSplitU"] == 1):
+                numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+                dataE = kw.vgprPool.checkOutAligned(int(numVgprs*self.cfg.gwvw), \
+                              int(ceil(numVgprs*self.cfg.gwvw)), "e data for ei=%u"%elementIdx, preventOverflow=False)
+            else:
+                dataE = 0
+            self.elementDataE.append(dataE)
+
+            if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                if coordOffset0 in scaleAVecVgprMap:
+                    dataScaleAVec = scaleAVecVgprMap[coordOffset0]
+                else:
+                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+                    dataScaleAVec = kw.vgprPool.checkOutAligned(int(numVgprs*self.cfg.gwvw), \
+                                  int(ceil(numVgprs*self.cfg.gwvw)), "scaleAVec data for ei=%u"%elementIdx, preventOverflow=False)
+                    scaleAVecVgprMap[coordOffset0] = dataScaleAVec
+                if coordOffset1 in scaleBVecVgprMap:
+                    dataScaleBVec = scaleBVecVgprMap[coordOffset1]
+                else:
+                    gwvw = min(self.cfg.gwvw, 2)
+                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+                    dataScaleBVec = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
+                                  int(ceil(numVgprs*gwvw)), "scaleBVec data for ei=%u"%elementIdx, preventOverflow=False)
+                    scaleBVecVgprMap[coordOffset1] = dataScaleBVec
+            else:
+                dataScaleAVec = 0
+                dataScaleBVec = 0
+            self.elementDataScaleAVec.append(dataScaleAVec)
+            self.elementDataScaleBVec.append(dataScaleBVec)
+            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                coordOffset = coordOffset0 if factorDim == 0 else coordOffset1
+                gwvw = self.cfg.gwvw if factorDim == 0 else min(self.cfg.gwvw, 2)
+                if coordOffset in scaleAlphaVecVgprMap:
+                    dataScaleAlphaVec = scaleAlphaVecVgprMap[coordOffset]
+                else:
+                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+                    dataScaleAlphaVec = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
+                                  int(ceil(numVgprs*gwvw)), "scaleAlphaVec data for ei=%u"%elementIdx, preventOverflow=False)
+                    scaleAlphaVecVgprMap[coordOffset] = dataScaleAlphaVec
+            else:
+                dataScaleAlphaVec = 0
+            self.elementDataScaleAlphaVec.append(dataScaleAlphaVec)
+            if batchElementSgprs != None:
+                if self.optSGPRUsage:
+                    mask = batchElementSgprs
+                else:
+                    mask = batchElementSgprs + self.cfg.numMaskSgprPerBatch + elementIdx * self.cfg.numMaskSgprPerElement
+                self.elementMask.append(mask)
+
+            #print "Edge=", edge, element
+            sumIdx = 0
+            if kernel["LocalSplitU"] > 1:
+                if len(self.elementSumIdx) == 0:
+                    sumIdx = kw.states.c.startVgprValu
+                else:
+                    sumIdx = self.elementSumIdx[-1] + self.cfg.numVgprPerValuC * self.cfg.gwvw
+            else:
+                bestVw                  = kernel["VectorWidthA"]
+                elementsLoadedPerVw     = kernel["NumThreads"] * bestVw
+                elementsLoadedPerbestVw = kernel["NumThreads"] * kernel["StoreVectorWidth"]
+
+                if elementsLoadedPerVw < elementsLoadedPerbestVw:
+                    bestVw = kernel["StoreVectorWidth"]
+
+                if kernel["EnableMatrixInstruction"]:
+                    alignment = self.cfg.numVgprPerValuC * self.cfg.gwvw
+                    sumIdx    = kw.vgprPool.checkOutAligned(self.cfg.numVgprPerValuC*self.cfg.gwvw, alignment, "vgprValuC") // self.cfg.numVgprPerValuC
+                else:
+                    sumIdx = kw.states.c.startVgprValu + vc0 + d0*kernel["VectorWidthA"] + vc1*kernel["ThreadTile0"] + d1*kernel["VectorWidthA"]*kernel["ThreadTile0"]
+            self.elementSumIdx.append(sumIdx) # sumIdx is an element idx, need to div/2 for half
+
             if self.optSingleColVgpr:
                 # use same address vgpr for all
                 addrEVgpr    = self.sharedColEVgprs
@@ -454,18 +647,26 @@ class StoreState:
                     addrEVgpr = None
                 #print ("d0=", d0, "vc0=", vc0, "elementCol=", elementCol)
 
-                if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and (kernel["GlobalSplitU"] == 1):
-                    addrScaleAVecVgpr = self.sharedColScaleAVecVgprs+elementCol
-                    addrScaleBVecVgpr = self.sharedColScaleBVecVgprs+elementCol
-                else:
-                    addrScaleAVecVgpr = None
-                    addrScaleBVecVgpr = None
-
                 if kernel["ProblemType"]["UseScaleAlphaVec"] and (kernel["GlobalSplitU"] == 1):
-                    addrScaleAlphaVecVgpr = self.sharedColScaleAlphaVecVgprs+elementCol
+                    if self.referenceVgprDim[self.factorDim] and self.referenceVgprDim[self.factorDim][0] == "ScaleAlpha":
+                        addrScaleAlphaVecVgpr = self.sharedColScaleAlphaVecVgprs+elementCol
+                    else:
+                        addrScaleAlphaVecVgpr = None
                 else:
                     addrScaleAlphaVecVgpr = None
 
+                if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and (kernel["GlobalSplitU"] == 1):
+                    if self.referenceVgprDim[0] and self.referenceVgprDim[0][0] == "ScaleA":
+                        addrScaleAVecVgpr = self.sharedColScaleAVecVgprs+elementCol
+                    else:
+                        addrScaleAVecVgpr = None
+                    if self.referenceVgprDim[1] and self.referenceVgprDim[1][0] == "ScaleB":
+                        addrScaleBVecVgpr = self.sharedColScaleBVecVgprs+elementCol
+                    else:
+                        addrScaleBVecVgpr = None
+                else:
+                    addrScaleAVecVgpr = None
+                    addrScaleBVecVgpr = None
             else:
                 # allocate new VGPR for each element:
                 addrDVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
@@ -485,149 +686,40 @@ class StoreState:
                         int(ceil(self.cfg.numVgprsPerAddr)), "loadBiasBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
                 else:
                     addrBiasVgpr = None
+
                 if kernel["ProblemType"]["UseE"] and (kernel["GlobalSplitU"] == 1):
                     addrEVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
                         int(ceil(self.cfg.numVgprsPerAddr)), "loadEBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
                 else:
                     addrEVgpr = None
 
+                if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                    if self.referenceVgprDim[self.factorDim] and self.referenceVgprDim[self.factorDim][0] == "ScaleAlpha":
+                        addrScaleAlphaVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
+                            int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAlphaVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
+                    else:
+                        addrScaleAlphaVecVgpr = None
+                else:
+                    addrScaleAlphaVecVgpr = None
+
                 if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                    addrScaleAVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
-                        int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
-                    addrScaleBVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
-                        int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
+                    if self.referenceVgprDim[0] and self.referenceVgprDim[0][0] == "ScaleA":
+                        addrScaleAVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
+                            int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
+                    else:
+                        addrScaleAVecVgpr = None
+                    if self.referenceVgprDim[1] and self.referenceVgprDim[1][0] == "ScaleB":
+                        addrScaleBVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
+                            int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
+                    else:
+                        addrScaleBVecVgpr = None
                 else:
                     addrScaleAVecVgpr = None
                     addrScaleBVecVgpr = None
-
-                if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                    addrScaleAlphaVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
-                        int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAlphaVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
-                else:
-                    addrScaleAlphaVecVgpr = None
             self.elementAddr.append(AddrCalculation(kw, self, addrCVgpr, addrDVgpr, addrGSUSyncVgprs, addrEVgpr, addrBiasVgpr, addrScaleAVecVgpr, addrScaleBVecVgpr, addrScaleAlphaVecVgpr, element, coordOffset0, \
-              self.kernelWriter.vgprs.coord1, coordOffset1, coordOffset1 - self.lastCoordOffset1, newCoord1))
-            # if numVgprsPerDataPerVI == 0.5, then two consecutive elements
-            # should have same data pointer, next should move.
-
-            if self.cfg.numVgprsPerDataPerVI > 0:
-                if self.cfg.halfDataRegPerVI:
-                    # TODO- check (H,H,H,H,S,S)
-                    if kernel["ProblemType"]["HighPrecisionAccumulate"] and \
-                       (kernel["ProblemType"]["DestDataType"].isBFloat16() or kernel["ProblemType"]["DestDataType"].isHalf()):
-                        data = kw.vgprPool.checkOutAligned(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
-                              int(ceil(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw))), "writeBatch-data for ei=%u and ei=%u"%(elementIdx,elementIdx+1), preventOverflow=not isOptNLL)
-                    else:
-                        if elementIdx%2 == 0:
-                            # allocate for two elements:
-                            data = kw.vgprPool.checkOutAligned(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
-                                   int(ceil(int(2*self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw))), "writeBatch-data for ei=%u and ei=%u"%(elementIdx,elementIdx+1), preventOverflow=not isOptNLL)
-                            lastData = data
-                        else:
-                            data = lastData
-                            del lastData
-                else:
-                    if self.cfg.numVgprsPerDataPerVI == 0.5 or self.cfg.numVgprsPerDataPerVI == 0.25:
-                        data = kw.vgprPool.checkOutAligned(int(ceil(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw)), \
-                              int(ceil(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw)), "writeBatch-data for ei=%u"%elementIdx, preventOverflow=False)
-                    else:
-                        data = kw.vgprPool.checkOutAligned(int(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
-                              int(ceil(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw)), "writeBatch-data for ei=%u"%elementIdx, preventOverflow=False)
-                    #data = kw.vgprPool.checkOut(int(self.cfg.numVgprsPerDataPerVI*self.cfg.gwvw), \
-                    #      "writeBatch-data for ei=%u"%elementIdx, preventOverflow=False)
-            else:
-                data = 0
-
-            self.elementData.append(data)
-
-            if self.useBias == DataDirection.READ:
-                coordOffset = coordOffset0 if factorDim == 0 else coordOffset1
-                if coordOffset in biasVgprMap:
-                    dataBias = biasVgprMap[coordOffset]
-                else:
-                    gwvw = self.cfg.gwvw if factorDim == 0 else min(self.cfg.gwvw, 2)
-                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                    dataBias = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
-                                int(ceil(numVgprs*gwvw)), "bias data for ei=%u"%elementIdx, preventOverflow=False)
-                    biasVgprMap[coordOffset] = dataBias
-            else:
-                dataBias = 0
-            self.elementDataBias.append(dataBias)
-
-            # Only needed in gradient activation
-            if (kernel["ProblemType"]["Gradient"] and kernel["ProblemType"]["ActivationType"] != 'none' and kernel["ProblemType"]["UseE"]) and (kernel["GlobalSplitU"] == 1):
-                numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                dataE = kw.vgprPool.checkOutAligned(int(numVgprs*self.cfg.gwvw), \
-                              int(ceil(numVgprs*self.cfg.gwvw)), "e data for ei=%u"%elementIdx, preventOverflow=False)
-            else:
-                dataE = 0
-            self.elementDataE.append(dataE)
-
-            if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                if coordOffset0 in scaleAVecVgprMap:
-                    dataScaleAVec = scaleAVecVgprMap[coordOffset0]
-                else:
-                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                    dataScaleAVec = kw.vgprPool.checkOutAligned(int(numVgprs*self.cfg.gwvw), \
-                                  int(ceil(numVgprs*self.cfg.gwvw)), "scaleAVec data for ei=%u"%elementIdx, preventOverflow=False)
-                    scaleAVecVgprMap[coordOffset0] = dataScaleAVec
-                if coordOffset1 in scaleBVecVgprMap:
-                    dataScaleBVec = scaleBVecVgprMap[coordOffset1]
-                else:
-                    gwvw = min(self.cfg.gwvw, 2)
-                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                    dataScaleBVec = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
-                                  int(ceil(numVgprs*gwvw)), "scaleBVec data for ei=%u"%elementIdx, preventOverflow=False)
-                    scaleBVecVgprMap[coordOffset1] = dataScaleBVec
-            else:
-                dataScaleAVec = 0
-                dataScaleBVec = 0
-            self.elementDataScaleAVec.append(dataScaleAVec)
-            self.elementDataScaleBVec.append(dataScaleBVec)
-
-            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
-                coordOffset = coordOffset0 if factorDim == 0 else coordOffset1
-                gwvw = self.cfg.gwvw if factorDim == 0 else (2 if self.cfg.gwvw >= 2 else 1)
-                if coordOffset in scaleAlphaVecVgprMap:
-                    dataScaleAlphaVec = scaleAlphaVecVgprMap[coordOffset]
-                else:
-                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                    dataScaleAlphaVec = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
-                                  int(ceil(numVgprs*gwvw)), "scaleAlphaVec data for ei=%u"%elementIdx, preventOverflow=False)
-                    scaleAlphaVecVgprMap[coordOffset] = dataScaleAlphaVec
-            else:
-                dataScaleAlphaVec = 0
-            self.elementDataScaleAlphaVec.append(dataScaleAlphaVec)
-
-            if batchElementSgprs != None:
-                if self.optSGPRUsage:
-                    mask = batchElementSgprs
-                else:
-                    mask = batchElementSgprs + self.cfg.numMaskSgprPerBatch + elementIdx * self.cfg.numMaskSgprPerElement
-                self.elementMask.append(mask)
-
-            #print "Edge=", edge, element
-            sumIdx = 0
-            if kernel["LocalSplitU"] > 1:
-                if len(self.elementSumIdx) == 0:
-                    sumIdx = kw.states.c.startVgprValu
-                else:
-                    sumIdx = self.elementSumIdx[-1] + self.cfg.numVgprPerValuC * self.cfg.gwvw
-            else:
-                bestVw                  = kernel["VectorWidthA"]
-                elementsLoadedPerVw     = kernel["NumThreads"] * bestVw
-                elementsLoadedPerbestVw = kernel["NumThreads"] * kernel["StoreVectorWidth"]
-
-                if elementsLoadedPerVw < elementsLoadedPerbestVw:
-                    bestVw = kernel["StoreVectorWidth"]
-
-                if kernel["EnableMatrixInstruction"]:
-                    alignment = self.cfg.numVgprPerValuC * self.cfg.gwvw
-                    sumIdx    = kw.vgprPool.checkOutAligned(self.cfg.numVgprPerValuC*self.cfg.gwvw, alignment, "vgprValuC") // self.cfg.numVgprPerValuC
-                else:
-                    sumIdx = kw.states.c.startVgprValu + vc0 + d0*kernel["VectorWidthA"] + vc1*kernel["ThreadTile0"] + d1*kernel["VectorWidthA"]*kernel["ThreadTile0"]
-            self.elementSumIdx.append(sumIdx) # sumIdx is an element idx, need to div/2 for half
+              self.kernelWriter.vgprs.coord1, coordOffset1, coordOffset1 - self.lastCoordOffset1, newCoord1, self.vectorDataTypes))
             self.lastCoordOffset1 = coordOffset1
+            
         # reset flag
         self.isReset = False
 
