@@ -171,6 +171,10 @@ class SignatureDefault(Signature):
         if kernel["ProblemType"]["Sparse"]:
             signature.addArg("MetaData", SVK.SIG_GLOBALBUFFER, "void" , "generic")
 
+        if kernel["StreamK"] > 0 and kernel["StreamKAtomic"] == 0:
+            signature.addArg("AddressWS", SVK.SIG_GLOBALBUFFER, dstValueType, "generic")
+            signature.addArg("AddressFlags", SVK.SIG_GLOBALBUFFER, dstValueType, "generic")
+
         for i in range(0, writer.states.d.numSgprStrides):
             signature.addArg(              "strideD%u"%i, SVK.SIG_VALUE,               "u32")
             userArgumentsInfo.gemmArgumentSize += 4
@@ -247,7 +251,7 @@ class SignatureDefault(Signature):
                 actValueType = 'pkf16'
             for name in kernel["ProblemType"]["ActivationType"].getAdditionalArgStringList():
                 signature.addArg(                   name, SVK.SIG_VALUE,        actValueType)
-            if kernel["ProblemType"]["ActivationType"] == 'all':
+            if kernel["ProblemType"]["ActivationType"] in ['all', 'hipblaslt_all'] :
                 signature.addArg(       "activationType", SVK.SIG_VALUE,               "u32")
 
         # TODO- combine one workspace
@@ -261,6 +265,23 @@ class SignatureDefault(Signature):
             signature.addArg(               "Synchronizer", SVK.SIG_GLOBALBUFFER, cptValueType, "generic")
             signature.addArg(               "GSUSync", SVK.SIG_VALUE,              "u32")
 
+        if kernel["StreamK"]:
+            # StreamK args
+            signature.addArg("MagicNumberProblemNumGroupTiles0",   SVK.SIG_VALUE, "u32")
+            signature.addArg("MagicShiftProblemNumGroupTiles0",    SVK.SIG_VALUE, "u32")
+            signature.addArg("ItersPerTile",                       SVK.SIG_VALUE, "u32")
+            signature.addArg("MagicNumberItersPerTile",            SVK.SIG_VALUE, "u32")
+            signature.addArg("MagicShiftItersPerTile",             SVK.SIG_VALUE, "u32")
+            signature.addArg("MagicNumProblemNumGroupTiles0By1",   SVK.SIG_VALUE, "u32")
+            signature.addArg("MagicShiftProblemNumGroupTiles0By1", SVK.SIG_VALUE, "u32")
+            signature.addArg("TotalIters",                         SVK.SIG_VALUE, "u32")
+            signature.addArg("SKItersPerWG",                       SVK.SIG_VALUE, "u32")
+            if kernel["StreamK"] >= 2: # Two-tile SK
+                signature.addArg("skGrid",                         SVK.SIG_VALUE, "u32")
+                signature.addArg("skTiles",                        SVK.SIG_VALUE, "u32")
+                signature.addArg("skExtraIters",                   SVK.SIG_VALUE, "u32")
+                # "dpTilesPerWG"
+
         activationType = ActivationType("all")
         for name in activationType.getAdditionalArgStringList():
             userArgumentsInfo.activationSize += userArgumentsInfo.actMaxSize
