@@ -29,8 +29,9 @@ from .KernelWriterBase import KernelWriterBase
 
 class KernelWriterActivationFunction(KernelWriterBase):
 
-  def __init__(self, state):
+  def __init__(self, state, cxxCompiler: str):
     super().__init__()
+    self.cxxCompiler = cxxCompiler
     self.state["ProblemType"] = deepcopy(state["ProblemType"])
     self.state["Kernel"] = state["Kernel"]
     self._tf = TensileInstructions()
@@ -79,7 +80,7 @@ class KernelWriterActivationFunction(KernelWriterBase):
 
     ptrStr = self.state["ProblemType"]["ActivationComputeDataType"].toDevice("HIP")
     names = ""
-    if self.state["ProblemType"]["ActivationType"] == 'all':
+    if self.state["ProblemType"]["ActivationType"] in ['all', 'hipblaslt_all']:
       names += ",\n"
       names += "  %s const activationType"%self.enumName
     for name in self.state["ProblemType"]["ActivationType"].getAdditionalArgStringList(False):
@@ -94,11 +95,11 @@ class KernelWriterActivationFunction(KernelWriterBase):
 
     isa = tuple(self.state["Kernel"]["ISA"])
     if not self._tf.isInit():
-      self._tf.init(isa, globalParameters["AssemblerPath"])
+      self._tf.init(isa, self.cxxCompiler)
     self._tf.setKernelInfo(isa, self.state["Kernel"]["WavefrontSize"])
 
     for arch in self.supportedArchs:
-      self._tf.init(arch, globalParameters["AssemblerPath"])
+      self._tf.init(arch, self.cxxCompiler)
       self._tf.setKernelInfo(arch, self.state["Kernel"]["WavefrontSize"])
       activationStrList.append(activation.generateInlineAssemblyBody(spaces, activationType))
 
@@ -138,10 +139,13 @@ class KernelWriterActivationFunction(KernelWriterBase):
     if self.state["ProblemType"]["ActivationType"] == 'none':
       return fileString
 
-    activationCDataType = self.state["ProblemType"]["ActivationComputeDataType"]
     isa = tuple(self.state["Kernel"]["ISA"])
-    self._tf.init(isa, globalParameters["AssemblerPath"])
+    self._tf.init(isa, self.cxxCompiler)
     self._tf.setKernelInfo(isa, self.state["Kernel"]["WavefrontSize"])
+
+    activationCDataType = self.state["ProblemType"]["ActivationComputeDataType"]
+    activationType = self.state["ProblemType"]["ActivationType"]
+    self._tf.setKernelInfo(tuple(self.state["Kernel"]["ISA"]), self.state["Kernel"]["WavefrontSize"])
     activation = ActivationInline(activationCDataType, not self.state["ProblemType"]["ActivationNoGuard"])
 
     fileString = "" # CHeader
@@ -154,8 +158,10 @@ class KernelWriterActivationFunction(KernelWriterBase):
     fileString += "#pragma clang diagnostic push\n"
     fileString += "#pragma clang diagnostic ignored \"-Winline-asm\"\n"
     fileString += self.functionSignature()
-    if self.state["ProblemType"]["ActivationType"] == 'all':
+    if activationType in ['all', 'hipblaslt_all']:
+      supportedBy = ActivationType.SupportedBy.ALL if activationType == 'all' else ActivationType.SupportedBy.HIPBLASLT
       for index, enumStr in enumerate(ActivationType.getEnumStrList(activationCDataType, \
+                                                                    supportedBy, \
                                                                     includeNone=False, \
                                                                     exportType=self.actExportType)):
         if index == 0:
