@@ -33,6 +33,7 @@ from contextlib import contextmanager
 import Tensile.TensileInstructions as ti
 from Tensile.Common import detectGlobalCurrentISA, restoreDefaultGlobalParameters, \
     assignGlobalParameters, getGfxName, gfxArch, globalParameters
+from Tensile.Utilities.Toolchain import ToolchainDefaults, validateToolchain
 
 def record_num_calls(f):
     @wraps(f)
@@ -675,7 +676,7 @@ if __name__ == '__main__':
     ap.add_argument('-o', '--output', type=str, required=True, help='Output path of compiled binary')
     ap.add_argument('-m', type=int, default=16, help='Dimension 0 of tile')
     ap.add_argument('-n', type=int, default=16, help='Dimension 1 of tile')
-    ap.add_argument('--toolchain', type=str, default='/opt/rocm/llvm/bin/clang++', help='Path to ROCm compiler')
+    ap.add_argument('--toolchain', type=str, default=ToolchainDefaults.CXX_COMPILER, help='Path to ROCm compiler')
     ap.add_argument('--debug-build', action='store_true', dest='debug_build', help='Build with debug information')
     ap.set_defaults(debug_build=False)
     ap.add_argument('--arch', type=str, default='gfx90a', help='Target architecture for assembler, e.g. gfx908. Default is gfx90a')
@@ -683,7 +684,7 @@ if __name__ == '__main__':
     output_path: str = args.output
     m: int = args.m
     n: int = args.n
-    toolchain_path: str = args.toolchain
+    toolchain_path: str = validateToolchain(args.toolchain)
     debug_build: bool = args.debug_build
     arch: str = args.arch
     isa = gfxArch(arch)
@@ -694,7 +695,7 @@ if __name__ == '__main__':
         detectGlobalCurrentISA()
         isa = globalParameters['CurrentISA']
         arch = getGfxName(isa)
-        toolchain_path = globalParameters['AssemblerPath']
+        toolchain_path = validateToolchain(ToolchainDefaults.CXX_COMPILER)
 
     ti.Base._global_ti.init(isa, toolchain_path, False)
     softmax = SoftmaxKernelGenerator(ti.DataType('S'), n, m, 256, arch)
