@@ -35,7 +35,7 @@
 
 #include <cstddef>
 
-namespace Tensile
+namespace TensileLite
 {
     namespace Client
     {
@@ -100,7 +100,9 @@ namespace Tensile
             DataType                         m_activationComputeType;
             std::vector<std::vector<size_t>> m_problemSizes;
             bool                             m_useUserArgs;
+            bool                             m_swizzleTensorA;
+            bool                             m_swizzleTensorB;
         };
 
     } // namespace Client
-} // namespace Tensile
+} // namespace TensileLite
