@@ -31,7 +31,7 @@
 #include <cstddef>
 #include <string>
 
-namespace Tensile
+namespace TensileLite
 {
     namespace Client
     {
@@ -128,6 +128,7 @@ namespace Tensile
             const std::string DeviceIndex         = "device-idx";
             const std::string FanSpeedRPMs        = "fan-rpm";
             const std::string HardwareSampleCount = "hardware-samples";
+            const std::string GfxFrequency        = "gfx-frequency(maximum)"; // GPU freq in Mhz
         }; // namespace ResultKey
 
         class ResultReporter : public RunListener
@@ -238,7 +239,11 @@ namespace Tensile
             }
             virtual void setNumWarmupRuns(size_t count) override {}
             virtual void preWarmup() override {}
-            virtual void postWarmup() override {}
+            virtual void postWarmup(TimingEvents const& startEvents,
+                                    TimingEvents const& stopEvents,
+                                    hipStream_t const&  stream) override
+            {
+            }
             virtual void validateWarmups(std::shared_ptr<ProblemInputs> inputs,
                                          TimingEvents const&            startEvents,
                                          TimingEvents const&            stopEvents) override
@@ -280,4 +285,4 @@ namespace Tensile
         };
 
     } // namespace Client
-} // namespace Tensile
+} // namespace TensileLite
