@@ -28,7 +28,7 @@
 
 #include <Tensile/Tensile.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     /**
  * \ingroup Hardware
@@ -69,6 +69,9 @@ namespace Tensile
             gfx1100 = 1100,
             gfx1101 = 1101,
             gfx1102 = 1102,
+            gfx1103 = 1103,
+            gfx1150 = 1150,
+            gfx1151 = 1151,
             gfx1200 = 1200,
             gfx1201 = 1201
         };
@@ -135,6 +138,18 @@ namespace Tensile
             {
                 return Processor::gfx1102;
             }
+            else if(archName.find("gfx1103") != std::string::npos)
+            {
+                return Processor::gfx1103;
+            }
+            else if(archName.find("gfx1150") != std::string::npos)
+            {
+                return Processor::gfx1150;
+            }
+            else if(archName.find("gfx1151") != std::string::npos)
+            {
+                return Processor::gfx1151;
+            }
             else if(archName.find("gfx1200") != std::string::npos)
             {
                 return Processor::gfx1200;
@@ -180,6 +195,12 @@ namespace Tensile
                 return "gfx1101";
             case AMDGPU::Processor::gfx1102:
                 return "gfx1102";
+            case AMDGPU::Processor::gfx1103:
+                return "gfx1103";
+            case AMDGPU::Processor::gfx1150:
+                return "gfx1150";
+            case AMDGPU::Processor::gfx1151:
+                return "gfx1151";
             case AMDGPU::Processor::gfx1200:
                 return "gfx1200";
             case AMDGPU::Processor::gfx1201:
@@ -198,6 +219,11 @@ namespace Tensile
         int         wavefrontSize    = 64;
         int         simdPerCu        = 4;
         int         computeUnitCount = 0;
+        int         skDynamicGrid    = 3;
+        int         skMaxCUs         = 0;
+        int         skGridMultiplier = 1;
+        int         skFixedGrid      = 0;
+        int         skFullTiles      = 1;
         std::string deviceName;
 
         virtual bool   runsKernelTargeting(Processor p) const;
@@ -213,6 +239,41 @@ namespace Tensile
 
         virtual std::string description() const;
 
+        const int getSKDynamicGrid() const
+        {
+            static const char* envStr = std::getenv("TENSILE_STREAMK_DYNAMIC_GRID");
+            static const int   value  = (envStr == NULL ? 3 : std::atoi(envStr));
+            return value;
+        }
+
+        const int getSKMaxCUs() const
+        {
+            static const char* envStr = std::getenv("TENSILE_STREAMK_MAX_CUS");
+            static const int   value  = (envStr == NULL ? 0 : std::atoi(envStr));
+            return value;
+        }
+
+        const int getSKGridMultiplier() const
+        {
+            static const char* envStr = std::getenv("TENSILE_STREAMK_GRID_MULTIPLIER");
+            static const int   value  = (envStr == NULL ? 1 : std::atoi(envStr));
+            return value;
+        }
+
+        const int getSKFixedGrid() const
+        {
+            static const char* envStr = std::getenv("TENSILE_STREAMK_FIXED_GRID");
+            static const int   value  = (envStr == NULL ? 0 : std::atoi(envStr));
+            return value;
+        }
+
+        const int getSKFullTiles() const
+        {
+            static const char* envStr = std::getenv("TENSILE_STREAMK_FULL_TILES");
+            static const int   value  = (envStr == NULL ? 1 : std::atoi(envStr));
+            return value;
+        }
+
         bool operator==(AMDGPU const& rhs) const
         {
             return processor == rhs.processor && computeUnitCount == rhs.computeUnitCount;
@@ -241,4 +302,4 @@ namespace Tensile
 
     TENSILE_API std::ostream& operator<<(std::ostream& stream, AMDGPU::Processor p);
     TENSILE_API std::ostream& operator<<(std::ostream& stream, AMDGPU g);
-} // namespace Tensile
+} // namespace TensileLite
