@@ -31,7 +31,7 @@
 #include <Tensile/ContractionProblem.hpp>
 #include <Tensile/TensorDescriptor_Detail.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     template <>
     struct Comparison<ContractionProblemGemm::FreeIndex>
@@ -140,16 +140,16 @@ namespace Tensile
                                         rhs.f32XdlMathOp());
         }
     };
-} // namespace Tensile
+} // namespace TensileLite
 
 namespace std
 {
     template <>
-    struct hash<Tensile::ContractionProblemGemm>
+    struct hash<TensileLite::ContractionProblemGemm>
     {
-        inline size_t operator()(Tensile::ContractionProblemGemm const& problem) const
+        inline size_t operator()(TensileLite::ContractionProblemGemm const& problem) const
         {
-            return Tensile::hash_combine(problem.operationIdentifier(),
+            return TensileLite::hash_combine(problem.operationIdentifier(),
                                          problem.a(),
                                          problem.b(),
                                          problem.c(),
@@ -178,15 +178,15 @@ namespace std
     };
 
     template <>
-    struct hash<std::vector<Tensile::ContractionProblemGemm>>
+    struct hash<std::vector<TensileLite::ContractionProblemGemm>>
     {
-        inline size_t operator()(std::vector<Tensile::ContractionProblemGemm> const& problems) const
+        inline size_t operator()(std::vector<TensileLite::ContractionProblemGemm> const& problems) const
         {
             size_t hash = 0;
             for(int idx = 0; idx < problems.size(); idx++)
             {
                 auto problem = problems[idx];
-                hash += Tensile::hash_combine(problem.operationIdentifier(),
+                hash += TensileLite::hash_combine(problem.operationIdentifier(),
                                               problem.a(),
                                               problem.b(),
                                               problem.c(),
