@@ -33,7 +33,7 @@
 
 #include <algorithm>
 
-namespace Tensile
+namespace TensileLite
 {
     // Which placeholder libraries should be initialized at start
     // To be extended in the future
@@ -59,6 +59,9 @@ namespace Tensile
         gfx1100,
         gfx1101,
         gfx1102,
+        gfx1103,
+        gfx1150,
+        gfx1151,
         gfx1200,
         gfx1201,
         All
@@ -109,6 +112,12 @@ namespace Tensile
             return "TensileLibrary_*_gfx1101";
         case LazyLoadingInit::gfx1102:
             return "TensileLibrary_*_gfx1102";
+        case LazyLoadingInit::gfx1103:
+            return "TensileLibrary_*_gfx1103";
+        case LazyLoadingInit::gfx1150:
+            return "TensileLibrary_*_gfx1150";
+        case LazyLoadingInit::gfx1151:
+             return "TensileLibrary_*_gfx1151";
         case LazyLoadingInit::gfx1200:
             return "TensileLibrary_*_gfx1200";
         case LazyLoadingInit::gfx1201:
@@ -303,4 +312,4 @@ namespace Tensile
         }
     };
 
-} // namespace Tensile
+} // namespace TensileLite
