@@ -32,7 +32,7 @@
 #include <Tensile/ContractionProblemPredicates.hpp>
 #include <Tensile/Predicates.hpp>
 
-namespace Tensile
+namespace TensileLite
 {
     namespace Serialization
     {
@@ -55,66 +55,75 @@ namespace Tensile
 
             static SubclassMap GetSubclasses()
             {
-                SubclassMap rv({
-                    Base::template Pair<Predicates::Contraction::Free0SizeMultiple>(),
-                    Base::template Pair<Predicates::Contraction::Free1SizeMultiple>(),
-                    Base::template Pair<Predicates::Contraction::BatchSizeMultiple>(),
-                    Base::template Pair<Predicates::Contraction::BatchSizeEqual>(),
-                    Base::template Pair<Predicates::Contraction::SynchronizerSizeCheck>(),
-                    Base::template Pair<Predicates::Contraction::BoundSizeMultiple>(),
-                    Base::template Pair<Predicates::Contraction::MaxProblemSizeGreaterThan>(),
-                    Base::template Pair<Predicates::Contraction::LeadingFree0SizesGreaterOrEqual>(),
-                    Base::template Pair<Predicates::Contraction::LeadingFree1SizesGreaterOrEqual>(),
-                    Base::template Pair<Predicates::Contraction::SizeEqual>(),
-                    Base::template Pair<Predicates::Contraction::SizeGreaterThan>(),
-                    Base::template Pair<Predicates::Contraction::SizeLessThan>(),
-                    Base::template Pair<Predicates::Contraction::SizeMultiple>(),
-                    Base::template Pair<Predicates::Contraction::StrideAEqual>(),
-                    Base::template Pair<Predicates::Contraction::StrideBEqual>(),
-                    Base::template Pair<Predicates::Contraction::StrideCEqual>(),
-                    Base::template Pair<Predicates::Contraction::StrideDEqual>(),
-                    Base::template Pair<Predicates::Contraction::LDCEqualsLDD>(),
-                    Base::template Pair<Predicates::Contraction::CEqualsD>(),
-                    Base::template Pair<Predicates::Contraction::AmaxDCheck>(),
-                    Base::template Pair<Predicates::Contraction::AlphaValue>(),
-                    Base::template Pair<Predicates::Contraction::BetaValue>(),
-                    Base::template Pair<Predicates::Contraction::BetaZero>(),
-                    Base::template Pair<Predicates::Contraction::BetaOne>(),
-                    Base::template Pair<Predicates::Contraction::HighPrecisionAccumulateEqual>(),
-                    Base::template Pair<Predicates::Contraction::KernelLanguageCompatible>(),
-                    Base::template Pair<Predicates::Contraction::DeterministicModeEqual>(),
-                    Base::template Pair<Predicates::Contraction::TypesEqual>(),
-                    Base::template Pair<Predicates::Contraction::OperationIdentifierEqual>(),
-                    Base::template Pair<Predicates::Contraction::BufferLoadOffsetLimitCheck>(),
-                    Base::template Pair<Predicates::Contraction::BufferLoadOffsetLimitCheck_Beta>(),
-                    Base::template Pair<Predicates::Contraction::BufferStoreOffsetLimitCheck>(),
-                    Base::template Pair<Predicates::Contraction::WorkspaceCheck>(),
-                    Base::template Pair<Predicates::Contraction::PersistentKernelCheck>(),
-                    Base::template Pair<Predicates::Contraction::GlobalSplitUCheckMinK>(),
-                    Base::template Pair<Predicates::Contraction::CDStridesEqual>(),
-                    Base::template Pair<Predicates::Contraction::StridedBatchedEqual>(),
-                    Base::template Pair<Predicates::Contraction::GroupedGemmEqual>(),
-                    Base::template Pair<Predicates::Contraction::CUEfficiency>(),
-                    Base::template Pair<Predicates::Contraction::Experimental>(),
-                    Base::template Pair<Predicates::Contraction::EqualityMatching>(),
-                    Base::template Pair<Predicates::Contraction::FreeSizeMatching>(),
-                    Base::template Pair<Predicates::Contraction::UseGradientEqual>(),
-                    Base::template Pair<Predicates::Contraction::ActivationCheck>(),
-                    Base::template Pair<Predicates::Contraction::ActivationComputeTypeEqual>(),
-                    Base::template Pair<Predicates::Contraction::ActivationNoGuardEqual>(),
-                    Base::template Pair<Predicates::Contraction::ActivationEnumWhiteList>(),
-                    Base::template Pair<Predicates::Contraction::UseBiasCheck>(),
-                    Base::template Pair<Predicates::Contraction::UseEEqual>(),
-                    Base::template Pair<Predicates::Contraction::UseScaleABCheck>(),
-                    Base::template Pair<Predicates::Contraction::UseScaleCDCheck>(),
-                    Base::template Pair<Predicates::Contraction::UseScaleAlphaVecCheck>(),
-                    Base::template Pair<Predicates::Contraction::BiasDataTypeWhiteList>(),
-                    Base::template Pair<Predicates::Contraction::BiasSrcWhiteList>(),
-                    Base::template Pair<Predicates::Contraction::SizeInRange>(),
-                    Base::template Pair<Predicates::Contraction::Sparse>(),
-                    Base::template Pair<Predicates::Contraction::F32XdlMathOpEqual>(),
-                    Base::template Pair<Predicates::Contraction::SupportDeviceUserArguments>(),
-                });
+                SubclassMap rv(
+                    {Base::template Pair<Predicates::Contraction::Free0SizeMultiple>(),
+                     Base::template Pair<Predicates::Contraction::Free1SizeMultiple>(),
+                     Base::template Pair<Predicates::Contraction::BatchSizeMultiple>(),
+                     Base::template Pair<Predicates::Contraction::BatchSizeEqual>(),
+                     Base::template Pair<Predicates::Contraction::SynchronizerSizeCheck>(),
+                     Base::template Pair<Predicates::Contraction::BoundSizeMultiple>(),
+                     Base::template Pair<Predicates::Contraction::MaxProblemSizeGreaterThan>(),
+                     Base::template Pair<
+                         Predicates::Contraction::LeadingFree0SizesGreaterOrEqual>(),
+                     Base::template Pair<
+                         Predicates::Contraction::LeadingFree1SizesGreaterOrEqual>(),
+                     Base::template Pair<Predicates::Contraction::SizeEqual>(),
+                     Base::template Pair<Predicates::Contraction::SizeGreaterThan>(),
+                     Base::template Pair<Predicates::Contraction::SizeLessThan>(),
+                     Base::template Pair<Predicates::Contraction::SizeMultiple>(),
+                     Base::template Pair<Predicates::Contraction::StrideAEqual>(),
+                     Base::template Pair<Predicates::Contraction::StrideBEqual>(),
+                     Base::template Pair<Predicates::Contraction::StrideCEqual>(),
+                     Base::template Pair<Predicates::Contraction::StrideDEqual>(),
+                     Base::template Pair<Predicates::Contraction::LDCEqualsLDD>(),
+                     Base::template Pair<Predicates::Contraction::CEqualsD>(),
+                     Base::template Pair<Predicates::Contraction::AIGreaterThanEqual>(),
+                     Base::template Pair<Predicates::Contraction::AILessThanEqual>(),
+                     Base::template Pair<Predicates::Contraction::AmaxDCheck>(),
+                     Base::template Pair<Predicates::Contraction::AlphaValue>(),
+                     Base::template Pair<Predicates::Contraction::BetaValue>(),
+                     Base::template Pair<Predicates::Contraction::BetaZero>(),
+                     Base::template Pair<Predicates::Contraction::BetaOne>(),
+                     Base::template Pair<Predicates::Contraction::HighPrecisionAccumulateEqual>(),
+                     Base::template Pair<Predicates::Contraction::KernelLanguageCompatible>(),
+                     Base::template Pair<Predicates::Contraction::DeterministicModeEqual>(),
+                     Base::template Pair<Predicates::Contraction::TypesEqual>(),
+                     Base::template Pair<Predicates::Contraction::OperationIdentifierEqual>(),
+                     Base::template Pair<Predicates::Contraction::BufferLoadOffsetLimitCheck>(),
+                     Base::template Pair<
+                         Predicates::Contraction::BufferLoadOffsetLimitCheck_Beta>(),
+                     Base::template Pair<Predicates::Contraction::BufferStoreOffsetLimitCheck>(),
+                     Base::template Pair<Predicates::Contraction::WorkgroupNumberCheck>(),
+                     Base::template Pair<Predicates::Contraction::WorkspaceCheck>(),
+                     Base::template Pair<Predicates::Contraction::PersistentKernelCheck>(),
+                     Base::template Pair<Predicates::Contraction::GlobalSplitUCheckMinK>(),
+                     Base::template Pair<Predicates::Contraction::CDStridesEqual>(),
+                     Base::template Pair<Predicates::Contraction::StridedBatchedEqual>(),
+                     Base::template Pair<Predicates::Contraction::GroupedGemmEqual>(),
+                     Base::template Pair<Predicates::Contraction::CUEfficiency>(),
+                     Base::template Pair<Predicates::Contraction::ExperimentalDTree>(),
+                     Base::template Pair<Predicates::Contraction::ExperimentalStreamK>(),
+                     Base::template Pair<Predicates::Contraction::EqualityMatching>(),
+                     Base::template Pair<Predicates::Contraction::FreeSizeMatching>(),
+                     Base::template Pair<Predicates::Contraction::UseGradientEqual>(),
+                     Base::template Pair<Predicates::Contraction::ActivationCheck>(),
+                     Base::template Pair<Predicates::Contraction::ActivationComputeTypeEqual>(),
+                     Base::template Pair<Predicates::Contraction::ActivationNoGuardEqual>(),
+                     Base::template Pair<Predicates::Contraction::ActivationEnumWhiteList>(),
+                     Base::template Pair<Predicates::Contraction::UseBiasCheck>(),
+                     Base::template Pair<Predicates::Contraction::UseEEqual>(),
+                     Base::template Pair<Predicates::Contraction::UseScaleABCheck>(),
+                     Base::template Pair<Predicates::Contraction::UseScaleCDCheck>(),
+                     Base::template Pair<Predicates::Contraction::UseScaleAlphaVecCheck>(),
+                     Base::template Pair<Predicates::Contraction::BiasDataTypeWhiteList>(),
+                     Base::template Pair<Predicates::Contraction::BiasSrcWhiteList>(),
+                     Base::template Pair<Predicates::Contraction::SizeInRange>(),
+                     Base::template Pair<Predicates::Contraction::Sparse>(),
+                     Base::template Pair<Predicates::Contraction::F32XdlMathOpEqual>(),
+                     Base::template Pair<Predicates::Contraction::SupportDeviceUserArguments>(),
+                     Base::template Pair<Predicates::Contraction::WorkgroupMappingXCCCheck>(),
+                     Base::template Pair<Predicates::Contraction::SwizzleTensorA>(),
+                     Base::template Pair<Predicates::Contraction::SwizzleTensorB>()});
 
                 auto gmap = Generic::GetSubclasses();
                 rv.insert(gmap.begin(), gmap.end());
@@ -246,6 +255,18 @@ namespace Tensile
         {
         };
 
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::AIGreaterThanEqual, IO>
+            : public AutoMappingTraits<Predicates::Contraction::AIGreaterThanEqual, IO>
+        {
+        };
+
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::AILessThanEqual, IO>
+            : public AutoMappingTraits<Predicates::Contraction::AILessThanEqual, IO>
+        {
+        };
+
         template <typename IO>
         struct MappingTraits<Predicates::Contraction::AmaxDCheck, IO>
             : public AutoMappingTraits<Predicates::Contraction::AmaxDCheck, IO>
@@ -324,6 +345,12 @@ namespace Tensile
         {
         };
 
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::WorkgroupNumberCheck, IO>
+            : public AutoMappingTraits<Predicates::Contraction::WorkgroupNumberCheck, IO>
+        {
+        };
+
         template <typename IO>
         struct MappingTraits<Predicates::Contraction::WorkspaceCheck, IO>
             : public AutoMappingTraits<Predicates::Contraction::WorkspaceCheck, IO>
@@ -367,8 +394,14 @@ namespace Tensile
         };
 
         template <typename IO>
-        struct MappingTraits<Predicates::Contraction::Experimental, IO>
-            : public AutoMappingTraits<Predicates::Contraction::Experimental, IO>
+        struct MappingTraits<Predicates::Contraction::ExperimentalDTree, IO>
+            : public AutoMappingTraits<Predicates::Contraction::ExperimentalDTree, IO>
+        {
+        };
+
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::ExperimentalStreamK, IO>
+            : public AutoMappingTraits<Predicates::Contraction::ExperimentalStreamK, IO>
         {
         };
 
@@ -487,10 +520,28 @@ namespace Tensile
         {
         };
 
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::SwizzleTensorA, IO>
+            : public AutoMappingTraits<Predicates::Contraction::SwizzleTensorA, IO>
+        {
+        };
+
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::SwizzleTensorB, IO>
+            : public AutoMappingTraits<Predicates::Contraction::SwizzleTensorB, IO>
+        {
+        };
+
         template <typename IO>
         struct MappingTraits<Predicates::Contraction::SupportDeviceUserArguments, IO>
             : public AutoMappingTraits<Predicates::Contraction::SupportDeviceUserArguments, IO>
         {
         };
+
+        template <typename IO>
+        struct MappingTraits<Predicates::Contraction::WorkgroupMappingXCCCheck, IO>
+            : public AutoMappingTraits<Predicates::Contraction::WorkgroupMappingXCCCheck, IO>
+        {
+        };
     } // namespace Serialization
-} // namespace Tensile
+} // namespace TensileLite
