@@ -53,6 +53,7 @@ class TensileInstructions:
         assemblerPath: str
         asmCaps: dict
         archCaps: dict
+        regCaps: dict
         asmBugs: dict
 
     @dataclass
@@ -68,9 +69,10 @@ class TensileInstructions:
             if isaVersion not in self._isaInfo: # type: ignore
                 asmCaps  = _initAsmCaps(isaVersion, assemblerPath, debug)
                 archCaps = _initArchCaps(isaVersion)
+                regCaps  = _initRegisterCaps(isaVersion, archCaps)
                 asmBugs  = _initAsmBugs(asmCaps)
                 self._isaInfo[isaVersion] = TensileInstructions.IsaInfo(assemblerPath, # type: ignore
-                    asmCaps, archCaps, asmBugs)
+                    asmCaps, archCaps, regCaps, asmBugs)
 
     def setDebugLevel(self, level: int) -> None:
         __TI_DEBUG_LEVEL__ = level
@@ -99,6 +101,9 @@ class TensileInstructions:
     def getArchCaps(self) -> dict:
         return self._isaInfo[self._kernelInfo[threading.get_ident()].isa].archCaps # type: ignore
 
+    def getRegCaps(self) -> dict:
+        return self._isaInfo[self._kernelInfo[threading.get_ident()].isa].regCaps
+
     def getAsmBugs(self) -> dict:
         return self._isaInfo[self._kernelInfo[threading.get_ident()].isa].asmBugs # type: ignore
 
@@ -148,6 +153,10 @@ class Item:
     @property
     def archCaps(self) -> dict:
         return _global_ti.getArchCaps()
+    
+    @property
+    def regCaps(self) -> dict:
+        return _global_ti.getRegCaps()
 
     @property
     def asmBugs(self) -> dict:
@@ -182,9 +191,9 @@ def getSlcBitName(hasGLCModifier):
   return "sc1"
 
 def getCOVFromParam(versionString):
-  if versionString == "default" or versionString == "V4":
+  if versionString == "default" or versionString == "V4" or versionString == "4":
     return 4
-  elif versionString == "V5":
+  elif versionString == "V5" or versionString == "5":
     return 5
   printExit("Unknown CodeObjectVersion %s" % (versionString))
 
@@ -204,7 +213,7 @@ def _tryAssembler(isaVersion: Tuple[int, int, int], assemblerPath: str, asmStrin
     if isaVersion[0] >= 10:
         options += ['-mwavefrontsize64']
 
-    args = [assemblerPath, '-x', 'assembler',
+    args = [str(assemblerPath), '-x', 'assembler',
             '-target', 'amdgcn-amdhsa',
             '-mcpu='+ getGfxName(isaVersion),
             *options,
@@ -236,8 +245,8 @@ def _initAsmCaps(isaVersion, assemblerPath, isDebug) -> dict:
     rv["HasExplicitCO"]     = _tryAssembler(isaVersion, assemblerPath, "v_add_co_u32 v0,vcc,v0,1", isDebug)
     rv["HasExplicitNC"]     = _tryAssembler(isaVersion, assemblerPath, "v_add_nc_u32 v0,v0,1", isDebug)
 
-    rv["HasDirectToLds"]    = _tryAssembler(isaVersion, assemblerPath, "buffer_load_dword v40, v36, s[24:27], s28 offen offset:0 lds", isDebug) \
-                                or _tryAssembler(isaVersion, assemblerPath, "buffer_load_b32 v40, v36, s[24:27], s28 offen offset:0 lds", isDebug)
+    rv["HasDirectToLds"]    = _tryAssembler(isaVersion, assemblerPath, "buffer_load_dword v36, s[24:27], s28 offen offset:0 lds", isDebug) \
+                                or _tryAssembler(isaVersion, assemblerPath, "buffer_load_b32 v36, s[24:27], s28 offen offset:0 lds", isDebug)
     rv["HasAddLshl"]        = _tryAssembler(isaVersion, assemblerPath, "v_add_lshl_u32 v47, v36, v34, 0x2", isDebug)
     rv["HasLshlOr"]         = _tryAssembler(isaVersion, assemblerPath, "v_lshl_or_b32 v47, v36, 0x2, v34", isDebug)
     rv["HasSMulHi"]         = _tryAssembler(isaVersion, assemblerPath, "s_mul_hi_u32 s47, s36, s34", isDebug)
@@ -326,6 +335,7 @@ def _initArchCaps(isaVersion) -> dict:
     rv["HasWave32"]          = isaVersion[0] in (10, 11, 12)
     rv["HasAccCD"]           = (isaVersion in [(9,0,10), (9,4,0), (9,4,1), (9,4,2)])
     rv["ArchAccUnifiedRegs"] = (isaVersion in [(9,0,10), (9,4,0), (9,4,1), (9,4,2)])
+    rv["CrosslaneWait"]      = (isaVersion in [(9,4,0), (9,4,1), (9,4,2)])
     rv["ForceStoreSC1"]      = (isaVersion in [(9,4,0), (9,4,1)])
     rv["TransOpWait"]        = (isaVersion in [(9,4,0), (9,4,1), (9,4,2)])
     rv["SDWAWait"]           = (isaVersion in [(9,4,0), (9,4,1), (9,4,2)])
@@ -333,9 +343,42 @@ def _initArchCaps(isaVersion) -> dict:
     rv["DSLow16NotPreserve"]       = isaVersion[0] == (12)
     rv["WrokGroupIdFromTTM"] = isaVersion[0] == (12)
     rv["NoSDWA"]             = isaVersion[0] == (12)
+    rv["VOP3ByteSel"]      = isaVersion[0] == (12)
     rv["HasFP8_OCP"]         = isaVersion[0] == (12)
     return rv
 
+def _initRegisterCaps(isaVersion, archCaps) -> dict:
+    rv = {}
+    rv["MaxVgpr"] = 256
+    # max allowed is 112 out of 112 , 6 is used by hardware 4 SGPRs are wasted
+    rv["MaxSgpr"] = 102
+
+    rv["PhysicalMaxVgpr"] = 512
+    rv["PhysicalMaxSgpr"] = 800
+
+    if isaVersion[0] == 10:
+        rv["PhysicalMaxVgprCU"] = 1024 * 32
+    elif isaVersion[0] == 11:
+        if isaVersion[2] == 2:
+            rv["PhysicalMaxVgprCU"] = 1024 * 32
+        else:
+            rv["PhysicalMaxVgprCU"] = 1536 * 32
+    elif isaVersion[0] == 12:
+        rv["PhysicalMaxVgprCU"] = 1536 * 32
+    elif isaVersion[0] == 9:
+        if archCaps["ArchAccUnifiedRegs"]:
+            rv["PhysicalMaxVgprCU"] = 2048 * 64
+        else:
+            rv["PhysicalMaxVgprCU"] = 1024 * 64
+    elif isaVersion[0] == 8:
+        rv["PhysicalMaxVgprCU"] = 1024 * 64
+    elif isaVersion[0] == 0:
+        rv["PhysicalMaxVgprCU"] = 0
+    else:
+        assert 0, "No valid VGPR value for this platform"
+
+    return rv
+
 def _initAsmBugs(asmCaps) -> dict:
     rv = {}
     rv["ExplicitCO"] = asmCaps["HasExplicitCO"]
