@@ -203,6 +203,12 @@ class Module(Item):
             return self.itemList.index(targetItem)
         return -1
 
+    def findIndexByType(self, targetType):
+        for i, item in enumerate(self.itemList):
+            if isinstance(item, targetType):
+                return i
+        return None
+
     def addComment(self, comment):
         """
         Convenience function to format arg as a comment and add TextBlock item
@@ -680,11 +686,17 @@ class _SignatureKernelDescriptor(Item):
             self.totalVgprs = self.accumOffset + totalAgprs
         else:
             self.accumOffset = None
-            self.totalVgprs = totalVgprs
+            self.totalVgprs = max(totalAgprs, totalVgprs)
         self.originalTotalVgprs = totalVgprs
         self.totalAgprs         = totalAgprs
         self.totalSgprs         = totalSgprs
 
+    def getNextFreeVgpr(self) -> int:
+        return self.totalVgprs
+    
+    def getNextFreeSgpr(self) -> int:
+        return self.totalSgprs
+
     def __str__(self):
         kdIndent = " " * 2
         kStr = ""
@@ -842,6 +854,12 @@ class SignatureBase(Item):
     def addDescription(self, text: str):
         self.descriptionList.append(TextBlock(slash(text)))
 
+    def getNextFreeVgpr(self) -> int:
+        return self.kernelDescriptor.getNextFreeVgpr()
+
+    def getNextFreeSgpr(self) -> int:
+        return self.kernelDescriptor.getNextFreeSgpr()
+
     def clearDescription(self, text: str):
         self.descriptionList = []
 
@@ -880,6 +898,12 @@ class KernelBody(Item):
         self.totalSgprs = totalSgprs
         self.signature.setGprs(totalVgprs=totalVgprs, totalAgprs=totalAgprs, \
             totalSgprs=totalSgprs)
+        
+    def getNextFreeVgpr(self) -> int:
+        return self.signature.getNextFreeVgpr()
+
+    def getNextFreeSgpr(self) -> int:
+        return self.signature.getNextFreeSgpr()
 
     def __str__(self) -> str:
         kStr = str(TextBlock(block3Line("Begin Kernel")))
