@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -301,7 +301,7 @@ class SaturateCastType(Enum):
     UPPER = 3
     LOWER = 4
 
-def VSaturateCastInt(sumIdxV, tmpVgpr, tmpSgpr, lowerBound, upperBound, type=SaturateCastType.NORMAL, initGpr=True):
+def VSaturateCastInt(vgprSumIdxV, tmpVgpr, tmpSgpr, lowerBound, upperBound, type=SaturateCastType.NORMAL, initGpr=True):
     # SaturateCastType = 0, normal case
     # SaturateCastType = 1, do nothing
     # SaturateCastType = 2, upperbound only
@@ -316,30 +316,30 @@ def VSaturateCastInt(sumIdxV, tmpVgpr, tmpSgpr, lowerBound, upperBound, type=Sat
             upperBoundHex = hex(upperBound)
             module.add(SMovkI32(dst=sgpr(tmpLowerBound), src=lowerBoundHex, comment="%d"%lowerBound ))
             module.add(VMovB32(dst=vgpr(tmpUpperBound), src=upperBoundHex, comment="%d"%upperBound ))
-        module.add(VMed3I32(dst=vgpr("ValuC+%u"%(sumIdxV)), src0=vgpr("ValuC+%u"%(sumIdxV)), src1=sgpr(tmpLowerBound), src2=vgpr(tmpUpperBound), comment="x= min(%d, max(%d, x))"%(upperBound, lowerBound)))
+        module.add(VMed3I32(dst=vgprSumIdxV, src0=vgprSumIdxV, src1=sgpr(tmpLowerBound), src2=vgpr(tmpUpperBound), comment="x= min(%d, max(%d, x))"%(upperBound, lowerBound)))
     elif type == SaturateCastType.DO_NOTHING:
         pass
     elif type == SaturateCastType.UPPER:
-        module.add(VMinI32(dst=vgpr("ValuC+%u"%(sumIdxV)), src0=upperBound, src1=vgpr("ValuC+%u"%(sumIdxV)), comment="x = min(%d, x)"%upperBound))
+        module.add(VMinI32(dst=vgprSumIdxV, src0=upperBound, src1=vgprSumIdxV, comment="x = min(%d, x)"%upperBound))
     elif type == SaturateCastType.LOWER:
-        module.add(VMaxI32(dst=vgpr("ValuC+%u"%(sumIdxV)), src0=lowerBound, src1=vgpr("ValuC+%u"%(sumIdxV)), comment="x = max(%d, x)"%lowerBound))
+        module.add(VMaxI32(dst=vgprSumIdxV, src0=lowerBound, src1=vgprSumIdxV, comment="x = max(%d, x)"%lowerBound))
     return module
 
 ########################################
 # Cvt
 ########################################
 
-def VCvtBF16toFP32(dst, src, vgprMask, vi):
+def VCvtBF16toFP32(dst, src, vgprMask, vi, additionalCmts=""):
     if (vi % 2) == 1:
-        return VAndB32(dst=vgpr(dst), src0=vgpr(src), src1=vgpr(vgprMask), comment="convert bf16 to fp32") # mask = hex(0xffff0000)
+        return VAndB32(dst=vgpr(dst), src0=vgpr(src), src1=vgpr(vgprMask), comment="cvt bf16 to fp32. " + additionalCmts) # mask = hex(0xffff0000)
     else:
-        return VLShiftLeftB32(dst=vgpr(dst), shiftHex=16, src=vgpr(src), comment="convert bf16 to fp32")
+        return VLShiftLeftB32(dst=vgpr(dst), shiftHex=16, src=vgpr(src), comment="cvt bf16 to fp32. " + additionalCmts)
 
 ########################################
 # init lds state
 ########################################
 def DSInit(tmpVgprRes: RegisterPoolResource, numThreads: int, \
-            ldsNumElements: int, numBytesPerElement: int, initValue):
+            ldsNumElements: int, initValue):
     assert tmpVgprRes.size > 1
     tmp = tmpVgprRes.idx
     tmpAddr = tmp + 1
@@ -350,7 +350,7 @@ def DSInit(tmpVgprRes: RegisterPoolResource, numThreads: int, \
     module.add(VMovB32(dst=vgpr(tmp), src=hex(initValue), comment="Init value"))
     module.add(VLShiftLeftB32(dst=vgpr(tmpAddr), shiftHex=2, src=vgpr("Serial"), \
                 comment="set per-thread address to init LDS"))
-    writesPerThread = ((ldsNumElements*numBytesPerElement-1)//numThreads//4) + 1
+    writesPerThread = ((ldsNumElements-1)//numThreads//4) + 1
     for i in range(0, writesPerThread):
         module.add(DSStoreB32(dstAddr=vgpr(tmpAddr), src=vgpr(tmp),
                     ds=DSModifiers(offset=(i*numThreads*4)), comment="init lds"))
